from smart_module.module_system.header_particle_systems import *
#psf_always_emit         = 0x0000000002
#psf_global_emit_dir     = 0x0000000010
#psf_emit_at_water_level = 0x0000000020
#psf_billboard_2d        = 0x0000000100 # up_vec = dir, front rotated towards camera
#psf_billboard_3d        = 0x0000000200 # front_vec point to camera.
#psf_turn_to_velocity    = 0x0000000400
#psf_randomize_rotation  = 0x0000001000
#psf_randomize_size      = 0x0000002000
#psf_2d_turbulance       = 0x0000010000

####################################################################################################################
#   Each particle system contains the following fields:
#  
#  1) Particle system id (string): used for referencing particle systems in other files.
#     The prefix psys_ is automatically added before each particle system id.
#  2) Particle system flags (int). See header_particle_systems.py for a list of available flags
#  3) mesh-name.
####
#  4) Num particles per second:    Number of particles emitted per second.
#  5) Particle Life:    Each particle lives this long (in seconds).
#  6) Damping:          How much particle's speed is lost due to friction.
#  7) Gravity strength: Effect of gravity. (Negative values make the particles float upwards.)
#  8) Turbulance size:  Size of random turbulance (in meters)
#  9) Turbulance strength: How much a particle is affected by turbulance.
####
# 10,11) Alpha keys :    Each attribute is controlled by two keys and 
# 12,13) Red keys   :    each key has two fields: (time, magnitude)
# 14,15) Green keys :    For example scale key (0.3,0.6) means 
# 16,17) Blue keys  :    scale of each particle will be 0.6 at the
# 18,19) Scale keys :    time 0.3 (where time=0 means creation and time=1 means end of the particle)
#
# The magnitudes are interpolated in between the two keys and remain constant beyond the keys.
# Except the alpha always starts from 0 at time 0.
####
# 20) Emit Box Size :   The dimension of the box particles are emitted from.
# 21) Emit velocity :   Particles are initially shot with this velocity.
# 22) Emit dir randomness
# 23) Particle rotation speed: Particles start to rotate with this (angular) speed (degrees per second).
# 24) Particle rotation damping: How quickly particles stop their rotation
####################################################################################################################

particle_systems = [
  
    ("game_rain", psf_billboard_2d|psf_global_emit_dir|psf_always_emit, "prtcl_rain",
     500, 0.5, 0.33, 1.0, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (1.0, 0.3), (1, 0.3),        #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (1.0, 1.0),   (1.0, 1.0),   #scale keys
     (8.2, 8.2, 0.2),           #emit box size
     (0, 0, -10.0),               #emit velocity
     0.0,                       #emit dir randomness
     0,                       #rotation speed
     0.5                        #rotation damping
    ),
    ("game_snow", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_snow_fall_1",
     150, 2, 0.2, 0.1, 30, 20,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.2, 1), (1, 1),        #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (1.0, 1.0),   (1.0, 1.0),   #scale keys
     (10, 10, 0.5),           #emit box size
     (0, 0, -5.0),               #emit velocity
     1,                       #emit dir randomness
     200,                       #rotation speed
     0.5                        #rotation damping
    ),
    
##    ("game_blood", psf_billboard_3d|psf_randomize_size|psf_randomize_rotation, "prtcl_dust_a",
##     50, 0.65, 0.95, 1.0, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
##     (0.3, 0.3), (1, 0.2),        #alpha keys
##     (1.0, 0.4), (1, 0.05),      #red keys
##     (1.0, 0.05),(1, 0.05),      #green keys
##     (1.0, 0.05),(1, 0.05),      #blue keys
##     (0.3, 0.5),   (1.0, 2.5),   #scale keys
##     (0.04, 0.01, 0.01),           #emit box size
##     (0, 1, 0.0),               #emit velocity
##     0.05,                       #emit dir randomness
##     0,                       #rotation speed
##     0.5                        #rotation damping
##    ),
##    
    ("game_blood", psf_billboard_3d |psf_randomize_size|psf_randomize_rotation,  "prt_mesh_blood_1",
     500, 0.65, 3, 0.5, 0, 0,        #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.0, 0.7), (0.7, 0.7),          #alpha keys
     (0.1, 0.7), (1, 0.7),      #red keys
     (0.1, 0.7), (1, 0.7),       #green keys
     (0.1, 0.7), (1, 0.7),      #blue keys
     (0.0, 0.015),   (1, 0.018),  #scale keys
     (0, 0.05, 0),               #emit box size
     (0, 1.0, 0.3),                #emit velocity
     0.9,                       #emit dir randomness
     0,                         #rotation speed
     0,                         #rotation damping
    ),
    ("game_blood_2", psf_billboard_3d | psf_randomize_size|psf_randomize_rotation ,  "prt_mesh_blood_3",
     2000, 0.6, 3, 0.3, 0, 0,        #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.0, 0.25), (0.7, 0.1),        #alpha keys
     (0.1, 0.7), (1, 0.7),      #red keys
     (0.1, 0.7), (1, 0.7),       #green keys
     (0.1, 0.7), (1, 0.7),      #blue keys
     (0.0, 0.15),   (1, 0.35),    #scale keys
     (0.01, 0.2, 0.01),             #emit box size
     (0.2, 0.3, 0),                 #emit velocity
     0.3,                         #emit dir randomness
     150,                       #rotation speed
     0,                       #rotation damping
     ),
    
 #   ("game_hoof_dust", psf_billboard_3d|psf_randomize_size|psf_randomize_rotation, "prtcl_dust_a",
 #    50, 1.0, 0.95, -0.1, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
 #    (0.0, 0.5), (1, 0.0),        #alpha keys
 #    (1.0, 0.9), (1, 0.05),      #red keys
 #    (1.0, 0.8),(1, 0.05),      #green keys
 #    (1.0, 0.7),(1, 0.05),      #blue keys
 #    (0.0, 7.5),   (1.0, 15.5),   #scale keys
 #    (0.2, 0.3, 0.2),           #emit box size
 #    (0, 0, 2.5),               #emit velocity
 #    0.05,                       #emit dir randomness
 #    100,                       #rotation speed
 #    0.5                        #rotation damping
 #   ),
     ("game_hoof_dust", psf_billboard_3d|psf_randomize_size|psf_randomize_rotation|psf_2d_turbulance, "prt_mesh_dust_1",#prt_mesh_dust_1
     5, 2.0,  10, 0.05, 10.0, 39.0, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.2, 0.5), (1, 0.0),        #alpha keys
     (0, 1), (1, 1),        #red keys
     (0, 0.9),(1, 0.9),         #green keys
     (0, 0.78),(1, 0.78),         #blue keys
     (0.0, 2.0),   (1.0, 3.5),   #scale keys
     (0.2, 0.3, 0.2),           #emit box size
     (0, 0, 3.9),                 #emit velocity
     0.5,                         #emit dir randomness
     130,                       #rotation speed
     0.5                        #rotation damping
    ),

    ("game_hoof_dust_snow", psf_billboard_3d|psf_randomize_size, "prt_mesh_snow_dust_1",#prt_mesh_dust_1
     6, 2, 3.5, 1, 10.0, 0.0, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.2, 1), (1, 1),        #alpha keys
     (0, 1), (1, 1),        #red keys
     (0, 1),(1, 1),         #green keys
     (0, 1),(1, 1),         #blue keys
     (0.5, 4),   (1.0, 5.7),   #scale keys
     (0.2, 1, 0.1),           #emit box size
     (0, 0, 1),                 #emit velocity
     2,                         #emit dir randomness
     0,                       #rotation speed
     0                        #rotation damping
    ),
     ("game_hoof_dust_mud", psf_billboard_2d|psf_randomize_size|psf_randomize_rotation|psf_2d_turbulance, "prt_mesh_mud_1",#prt_mesh_dust_1
     5, .7,  10, 3, 0, 0, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 1), (1, 1),        #alpha keys
     (0, .7), (1, .7),        #red keys
     (0, 0.6),(1, 0.6),         #green keys
     (0, 0.4),(1, 0.4),         #blue keys
     (0.0, 0.2),   (1.0, 0.22),   #scale keys
     (0.15, 0.5, 0.1),           #emit box size
     (0, 0, 15),                 #emit velocity
     6,                         #emit dir randomness
     200,                       #rotation speed
     0.5                        #rotation damping
    ),

    ("game_water_splash_1", psf_billboard_3d|psf_randomize_size|psf_randomize_rotation|psf_emit_at_water_level, "prtcl_drop",
     20, 0.85, 0.25, 0.9, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.3, 0.5), (1, 0.0),        #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (0.0, 0.3),   (1.0, 0.18),   #scale keys
     (0.3, 0.2, 0.1),           #emit box size
     (0, 1.2, 2.3),               #emit velocity
     0.3,                       #emit dir randomness
     50,                       #rotation speed
     0.5                        #rotation damping
    ),
    
    ("game_water_splash_2", psf_billboard_3d|psf_randomize_size|psf_randomize_rotation|psf_emit_at_water_level, "prtcl_splash_b",
     30, 0.4, 0.7, 0.5, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.3, 1.0), (1, 0.3),        #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (0.0, 0.25),   (1.0, 0.7),   #scale keys
     (0.4, 0.3, 0.1),           #emit box size
     (0, 1.3, 1.1),               #emit velocity
     0.1,                       #emit dir randomness
     50,                       #rotation speed
     0.5                        #rotation damping
    ),

    ("game_water_splash_3", psf_emit_at_water_level , "prt_mesh_water_wave_1",
     5, 2.0, 0, 0.0, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.03, 0.2), (1, 0.0),        #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (0.0, 3),   (1.0, 10),   #scale keys
     (0.0, 0.0, 0.0),           #emit box size
     (0, 0, 0),                 #emit velocity
     0.0,                       #emit dir randomness
     0,                       #rotation speed
     0.5                        #rotation damping
    ),

    ("torch_fire", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     50, 0.35, 0.2, 0.03, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.8), (1, 0),        #alpha keys
     (0.5, 1.0), (1, 0.9),      #red keys
     (0.5, 0.7),(1, 0.3),       #green keys
     (0.5, 0.2), (1, 0.0),      #blue keys
     (0, 0.15),   (0.4, 0.3),   #scale keys
     (0.04, 0.04, 0.01),      #emit box size
     (0, 0, 0.5),               #emit velocity
     0.0,                       #emit dir randomness
     200,                       #rotation speed
     0.5                        #rotation damping
    ),
    ("fire_glow_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prt_mesh_fire_2",
     2, 0.55, 0.2, 0.0, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.9), (1, 0),          #alpha keys
     (0.0, 0.9), (1, 0.9),      #red keys
     (0.0, 0.7),(1, 0.7),       #green keys
     (0.0, 0.4), (1, 0.4),      #blue keys
     (0, 2),   (1.0, 2),        #scale keys
     (0.0, 0.0, 0.0),           #emit box size
     (0, 0, 0),                 #emit velocity
     0.0,                       #emit dir randomness
     0,
     0
    ),

    ("fire_glow_fixed", psf_billboard_3d|psf_global_emit_dir, "prt_mesh_fire_2",
     4, 100.0, 0.2, 0.0, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (-0.01, 1.0), (1, 1.0),          #alpha keys
     (0.0, 0.9), (1, 0.9),      #red keys
     (0.0, 0.7),(1, 0.7),       #green keys
     (0.0, 0.4), (1, 0.4),      #blue keys
     (0, 2),   (1.0, 2),        #scale keys
     (0.0, 0.0, 0.0),           #emit box size
     (0, 0, 0),                 #emit velocity
     0.0,                       #emit dir randomness
     0,
     0
    ),

    ("torch_smoke", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prtcl_dust_a",
     15, 0.5, 0.2, -0.2, 10.0, 0.1,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.25), (1, 0),       #alpha keys
     (0.0, 0.2), (1, 0.1),      #red keys
     (0.0, 0.2),(1, 0.09),      #green keys
     (0.0, 0.2), (1, 0.08),     #blue keys
     (0, 0.5),   (0.8, 2.5),    #scale keys
     (0.1, 0.1, 0.1),           #emit box size
     (0, 0, 1.5),               #emit velocity
     0.1                        #emit dir randomness
    ),
     ("flue_smoke_short", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prtcl_dust_a",
     15, 1.5, 0.1, -0.0, 10.0, 12, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.0, 0.3), (1, 0),        #alpha keys
     (0.0, 0.2), (1, 0.1),      #red keys
     (0.0, 0.2),(1, 0.09),      #green keys
     (0.0, 0.2), (1, 0.08),     #blue keys
     (0, 1.5),   (1, 7),          #scale keys
     (0, 0, 0),           #emit box size
     (0, 0, 1.5),               #emit velocity
     0.1,                        #emit dir randomness
     150,
     0.8,
    ),
    ("flue_smoke_tall", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prtcl_dust_a",
     15, 3, 0.5, -0.0, 15.0, 12,#num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.35), (1, 0),         #alpha keys
     (0.0, 0.3), (1, 0.1),      #red keys
     (0.0, 0.3),(1, 0.1),       #green keys
     (0.0, 0.3), (1, 0.1),      #blue keys
     (0, 2),   (1, 7),        #scale keys
     (0, 0, 0),                 #emit box size
     (0, 0, 1.5),               #emit velocity
     0.1,                       #emit dir randomness
     150,
     0.5,
    ),

    ("war_smoke_tall", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prt_mesh_smoke_1",
     5, 12, 0, 0, 7, 7, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.25), (1, 0),          #alpha keys
     (0.0, 1), (1, 0.8),      #red keys
     (0.0, 1),(1, 0.8),      #green keys
     (0.0, 1), (1, 0.8),     #blue keys
     (0, 2.2),   (1, 15),        #scale keys
     (0, 0, 0),                 #emit box size
     (0, 0, 2.2),               #emit velocity
     0.1,                        #emit dir randomness
     100,
     0.2,
    ),
    
     ("ladder_dust_6m", psf_billboard_3d, "prt_mesh_smoke_1",
     700, 0.9, 0, 0, 7, 7, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.25), (1, 0),          #alpha keys
     (0.0, 1), (1, 0.8),      #red keys
     (0.0, 1),(1, 0.8),      #green keys
     (0.0, 1), (1, 0.8),     #blue keys
     (0, 1),   (1, 2),        #scale keys
     (0.75, 0.75, 3.5),                 #emit box size  (6.5 is equal to (12m / 2) + 0.5)
     (0, 0, 0),               #emit velocity
     0.1,                        #emit dir randomness
     100,
     0.2,
    ),

     ("ladder_dust_8m", psf_billboard_3d, "prt_mesh_smoke_1",
     900, 0.9, 0, 0, 7, 7, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.25), (1, 0),          #alpha keys
     (0.0, 1), (1, 0.8),      #red keys
     (0.0, 1),(1, 0.8),      #green keys
     (0.0, 1), (1, 0.8),     #blue keys
     (0, 1),   (1, 2),        #scale keys
     (0.75, 0.75, 4.5),                 #emit box size  (6.5 is equal to (12m / 2) + 0.5)
     (0, 0, 0),               #emit velocity
     0.1,                        #emit dir randomness
     100,
     0.2,
    ),

     ("ladder_dust_10m", psf_billboard_3d, "prt_mesh_smoke_1",
     1100, 0.9, 0, 0, 7, 7, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.25), (1, 0),          #alpha keys
     (0.0, 1), (1, 0.8),      #red keys
     (0.0, 1),(1, 0.8),      #green keys
     (0.0, 1), (1, 0.8),     #blue keys
     (0, 1),   (1, 2),        #scale keys
     (0.75, 0.75, 5.5),                 #emit box size  (6.5 is equal to (12m / 2) + 0.5)
     (0, 0, 0),               #emit velocity
     0.1,                        #emit dir randomness
     100,
     0.2,
    ),

     ("ladder_dust_12m", psf_billboard_3d, "prt_mesh_smoke_1",
     1300, 0.9, 0, 0, 7, 7, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.25), (1, 0),          #alpha keys
     (0.0, 1), (1, 0.8),      #red keys
     (0.0, 1),(1, 0.8),      #green keys
     (0.0, 1), (1, 0.8),     #blue keys
     (0, 1),   (1, 2),        #scale keys
     (0.75, 0.75, 6.5),                 #emit box size  (6.5 is equal to (12m / 2) + 0.5)
     (0, 0, 0),               #emit velocity
     0.1,                        #emit dir randomness
     100,
     0.2,
    ),

     ("ladder_dust_14m", psf_billboard_3d, "prt_mesh_smoke_1",
     1500, 0.9, 0, 0, 7, 7, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 0.25), (1, 0),          #alpha keys
     (0.0, 1), (1, 0.8),      #red keys
     (0.0, 1),(1, 0.8),      #green keys
     (0.0, 1), (1, 0.8),     #blue keys
     (0, 1),   (1, 2),        #scale keys
     (0.75, 0.75, 7.5),                 #emit box size  (7.5 is equal to (14m / 2) + 0.5)
     (0, 0, 0),               #emit velocity
     0.1,                        #emit dir randomness
     100,
     0.2,
    ),

    ("ladder_straw_6m", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     700, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.3),   (1, 0.3),    #scale keys
     (0.75, 0.75, 3.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    ("ladder_straw_8m", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     900, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.3),   (1, 0.3),    #scale keys
     (0.75, 0.75, 4.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    ("ladder_straw_10m", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     1100, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.3),   (1, 0.3),    #scale keys
     (0.75, 0.75, 5.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    ("ladder_straw_12m", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     1300, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.3),   (1, 0.3),    #scale keys
     (0.75, 0.75, 6.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    ("ladder_straw_14m", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     1500, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.3),   (1, 0.3),    #scale keys
     (0.75, 0.75, 7.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    ("torch_fire_sparks", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size,  "prt_sparks_mesh_1",
     10, 0.7, 0.2, 0, 10.0, 0.02,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.66, 1), (1, 0),          #alpha keys
     (0.1, 0.7), (1, 0.7),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.1), (1, 0.1),      #blue keys
     (0.1, 0.05),   (1, 0.05),  #scale keys
     (0.1, 0.1, 0.1),           #emit box size
     (0, 0, 0.9),               #emit velocity
     0.0,                       #emit dir randomness
     0,
     0,
    ),

    ("fire_sparks_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size,  "prt_sparks_mesh_1",
     10, 1.5, 0.2, 0, 3, 10,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.6, 1), (1, 1),          #alpha keys
     (0.1, 0.7), (1, 0.7),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.1), (1, 0.1),      #blue keys
     (0.1, 0.07),   (1, 0.03),    #scale keys
     (0.17, 0.17, 0.01),           #emit box size
     (0, 0, 1),                 #emit velocity
     0.0,                       #emit dir randomness
     0,
     0,
    ),
    
    ("pistol_smoke", psf_billboard_3d, "prtcl_dust_a",
     90, 2.5, 0.6, -0.2, 60.0, 1.5,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.0, 0.75), (1, 0),       #alpha keys
     (0.0, 0.7), (1, 0.4),      #red keys
     (0.0, 0.7),(1, 0.4),       #green keys
     (0.0, 0.7), (1, 0.4),      #blue keys
     (0, 1.5),   (0.5, 11.0),   #scale keys
     (0.1, 0.1, 0.1),           #emit box size
     (2, 2, 0),                 #emit velocity
     0.1                        #emit dir randomness
    ),
    
#    ("cooking_fire", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prtcl_fire",
#    50, 0.5, 0.2, -0.05, 30.0, 0.3,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
#    (0.5, 1), (1, 1),          #alpha keys
#     (0.5, 1.0), (1, 0.9),     #red keys
#     (0.5, 0.4), (1, 0.1),     #green keys
#     (0.5, 0.2), (1, 0.0),     #blue keys
#     (0.3, 0.9),   (0.9, 2),   #scale keys
#     (0.07, 0.07, 0.01),       #emit box size
#     (0, 0, 0.1),              #emit velocity
#     0.1                       #emit dir randomness
#    ),

     ("brazier_fire_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     25, 0.5, 0.1, 0.0, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.4), (1.0, 0),        #alpha keys
     (0.5, 1.0), (1.0, 0.9),      #red keys
     (0.5, 0.7),(1.0, 0.3),       #green keys
     (0.5, 0.2), (1, 0.0),      #blue keys
     (0.1, 0.2),   (1.0, 0.5),      #scale keys
     (0.1, 0.1, 0.01),        #emit box size
     (0.0, 0.0, 0.4),                 #emit velocity
     0.0,                       #emit dir randomness
     100,                       #rotation speed
     0.2                        #rotation damping
     ),
#              ("cooking_smoke", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prt_mesh_steam_1",
#     3, 3.5, 0.4, -0.03, 10.0, 10.9,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
#     (0.4, 1), (1, 0),          #alpha keys
 #    (0.0, 0.6), (1, 0.3),      #red keys
#     (0.0, 0.6),(1, 0.3),       #green keys
#     (0.0, 0.6), (1, 0.3),      #blue keys
#     (0, 2.5),   (0.9, 7.5),    #scale keys
 #    (0.1, 0.1, 0.06),          #emit box size
 #    (0, 0, 1.3),               #emit velocity
#     0.2,                       #emit dir randomness
 #    200,                       #rotation speed
 #    0.2,                       #rotation damping
 #   ),

    ("cooking_fire_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     25, 0.35, 0.1, 0.03, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.8), (1, 0),        #alpha keys
     (0.5, 0.5*1.0), (1, 0.3*0.9),      #red keys
     (0.5, 0.5*0.7),(1, 0.3*0.3),       #green keys
     (0.5, 0.5*0.2), (1, 0.0),      #blue keys
     (0.1, 0.5),   (1, 1),      #scale keys
     (0.05, 0.05, 0.01),        #emit box size
     (0, 0, 1),                 #emit velocity
     0.0,                       #emit dir randomness
     200,                       #rotation speed
     0.0                        #rotation damping
     ),
    
    ("cooking_smoke", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prt_mesh_smoke_1",
     4, 4, 0.1, 0, 3, 5, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.2, 0.20), (1.0, 0.0),          #alpha keys
     (0.0, 0.8), (1.0, 1.0),      #red keys
     (0.0, 0.8),(1.0, 1.0),      #green keys
     (0.0, 0.85), (1.0, 1.0),     #blue keys
     (0.0, 0.65),   (1.0, 3.0),        #scale keys
     (0.0, 0.0, 0.0),           #emit box size
     (0.0, 0.0, 1.2),               #emit velocity
     0.0,                        #emit dir randomness
     0,
     0,
    ),

    ("food_steam", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prt_mesh_steam_1",
     3, 1, 0, 0, 8.0, 1,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.1), (1, 0),        #alpha keys
     (0.0, 1), (1, 0.1),        #red keys
     (0.0, 1),(1, 0.1),         #green keys
     (0.0, 1), (1, 0.1),        #blue keys
     (0, 0.2),   (0.9, 0.5),      #scale keys
     (0.05, 0.05, 0),          #emit box size
     (0, 0, 0.1),               #emit velocity
     0,                         #emit dir randomness
     100,                       #rotation speed
     0.5,                       #rotation damping
    ),

    ("candle_light", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prt_mesh_candle_fire_1",
     7, 1.1, 0.6, -0.0, 10.0, 0.2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.5), (1, 0),        #alpha keys
     (0.5, 1.0), (1, 0.9),      #red keys
     (0.5, 0.6), (1, 0.1),      #green keys
     (0.5, 0.2), (1, 0.0),      #blue keys
     (0.3, 0.2),   (1, 0.0),    #scale keys
     (0.0, 0.0, 0.0),           #emit box size
     (0, 0, 0.09),              #emit velocity
      0,                        #emit dir randomness
      0,                        #rotation speed
      0                         #rotation damping
    ),
    ("candle_light_small", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prt_mesh_candle_fire_1",
     4, 1.1, 0.6, -0.0, 10.0, 0.2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.8), (1, 0),        #alpha keys
     (0.5, 1.0), (1, 0.9),      #red keys
     (0.5, 0.6), (1, 0.1),      #green keys
     (0.5, 0.2), (1, 0.0),      #blue keys
     (0.3, 0.13),   (1, 0.0),   #scale keys
     (0.0, 0.0, 0.0),           #emit box size
     (0, 0, 0.06),              #emit velocity
      0,                        #emit dir randomness
      0,                        #rotation speed
      0                         #rotation damping
    ),
    ("lamp_fire", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_rotation|psf_randomize_size, "prt_mesh_fire_1",
     10, 0.8, 0.6, -0.0, 10.0, 0.4,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.5), (1, 0),        #alpha keys
     (0.5, 1.0), (1, 0.9),      #red keys
     (0.5, 0.8), (1, 0.1),      #green keys
     (0.5, 0.4), (1, 0.0),      #blue keys
     (0.3, 0.35), (0.9, 0.5),    #scale keys
     (0.01, 0.01, 0.0),         #emit box size
     (0, 0, 0.35),               #emit velocity
     0.03,                      #emit dir randomness
     100,                       #rotation speed
     0.5,                       #rotation damping
    ),
#*-*-*-*-* D U M M Y *-*-*-*-#
    ("dummy_smoke", psf_billboard_3d|psf_randomize_size,  "prt_mesh_dust_1",
     500, 3, 15, -0.05, 10.0, 0.2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.5), (1, 0),        #alpha keys
     (0.1, 0.8), (1, 0.8),      #red keys
     (0.1, 0.7),(1, 0.7),       #green keys
     (0.1, 0.6), (1, 0.7),      #blue keys
     (0.0, 0.7),   (1, 2.2),    #scale keys
     (0.2, 0.2, 0.5),           #emit box size
     (0, 0, 0.05),               #emit velocity
     2,                         #emit dir randomness
     10,                        #rotation speed
     0.1,                       #rotation damping
    ),

    ("dummy_straw", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     500, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.3),   (1, 0.3),    #scale keys
     (0.2, 0.2, 0.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    ("dummy_smoke_big", psf_billboard_3d|psf_randomize_size,  "prt_mesh_dust_1",
     500, 9, 15, -0.05, 10.0, 0.2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.9), (1, 0),        #alpha keys
     (0.1, 0.8), (1, 0.8),      #red keys
     (0.1, 0.7),(1, 0.7),       #green keys
     (0.1, 0.6), (1, 0.7),      #blue keys
     (0.0, 5),   (1, 15.0),    #scale keys
     (3, 3, 5),           #emit box size
     (0, 0, 0.05),               #emit velocity
     2,                         #emit dir randomness
     10,                        #rotation speed
     0.1,                       #rotation damping
    ),

    ("dummy_straw_big", psf_randomize_size | psf_randomize_rotation,  "prt_mesh_straw_1",
     500, 3, 2, 2.0, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 0.8),   (1, 0.8),    #scale keys
     (3, 3, 3),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

#*-*-*-*-* D U M M Y  E N D *-*-*-*-#
#*-*-*-*-* GOURD *-*-*-*-#
    ("gourd_smoke", psf_billboard_3d|psf_randomize_size,  "prt_mesh_dust_1",
     500, 3, 15, -0.05, 10.0, 0.2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.5), (1, 0),        #alpha keys
     (0.1, 0.8), (1, 0.8),      #red keys
     (0.1, 0.7),(1, 0.7),       #green keys
     (0.1, 0.6), (1, 0.7),      #blue keys
     (0.0, 0.5),   (1, 1),    #scale keys
     (0.2, 0.2, 0.5),           #emit box size
     (0, 0, 0.05),               #emit velocity
     2,                         #emit dir randomness
     10,                        #rotation speed
     0.1,                       #rotation damping
    ),

    ("gourd_piece_1", psf_randomize_rotation,  "prt_gourd_piece_1",
     15, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 1),   (1, 1),    #scale keys
     (0.2, 0.2, 0.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),
    
    ("gourd_piece_2", psf_randomize_size | psf_randomize_rotation,  "prt_gourd_piece_2",
     50, 1, 2, 0.9, 10, 2,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 1), (1, 1),          #alpha keys
     (0.1, 0.6), (1, 0.6),      #red keys
     (0.1, 0.5),(1, 0.5),       #green keys
     (0.1, 0.4), (1, 0.4),      #blue keys
     (0.0, 1),   (1, 1),    #scale keys
     (0.2, 0.2, 0.5),           #emit box size
     (0, 0, 0),                 #emit velocity
     2.3,                       #emit dir randomness
     200,                       #rotation speed
     0,                       #rotation damping
    ),

    
##    ("rat_particle", psf_global_emit_dir|psf_2d_turbulance | psf_randomize_size |psf_billboard_3d,  "rat_particle",
##     500, 4, 0, 0, 20, 10,      #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
##     (0.1, 1), (1, 1),          #alpha keys
##     (0.1, 0.6), (1, 0.6),      #red keys
##     (0.1, 0.5),(1, 0.5),       #green keys
##     (0.1, 0.4), (1, 0.4),      #blue keys
##     (0.1, 1),   (1, 1),        #scale keys
##     (0.1, 0.1, 0.1),           #emit box size
##     (0, 0, 0),                 #emit velocity
##     5,                         #emit dir randomness
##    ),

#*-*-*-**** BLOOD ****-*-*-*#
    
##("blood_hit_1", psf_billboard_3d | psf_randomize_size ,  "prt_mesh_blood_1",
##     5000, 0.5, 6, 0.5, 0, 0,        #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
##     (0.1, 1), (1, 0),          #alpha keys
##     (0.1, 0.6), (1, 0.6),      #red keys
##     (0.1, 0.5),(1, 0.5),       #green keys
##     (0.1, 0.4), (1, 0.4),      #blue keys
##     (0.0, 0.05),   (1, 0.05),  #scale keys
##     (0, 0.5, 0),               #emit box size
##     (0, -1, 0),                #emit velocity
##     1.5,                       #emit dir randomness
##     0,                         #rotation speed
##     0,                         #rotation damping
##    ),
##    #("blood_hit_2", 0 ,  "prt_mesh_blood_2",
##    # 500, 0.3, 0,0, 0, 0,       #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
##    # (0.1, 1), (1, 0.0),        #alpha keys
##    # (0.1, 0.6), (1, 0.6),      #red keys
##    # (0.1, 0.5),(1, 0.5),       #green keys
##    # (0.1, 0.4), (1, 0.4),      #blue keys
##    # (0.0, 0.4),   (1, 2),      #scale keys
##    # (0.0, 0.0, 0.0),           #emit box size
##    # (0, -0.1, 0),              #emit velocity
##    # 0,                         #emit dir randomness
##    # 0,                         #rotation speed
##    # 0,                         #rotation damping
##    # ),
##    ("blood_hit_3", psf_billboard_3d | psf_randomize_size ,  "prt_mesh_blood_3",
##     500, 0.3, 1,0.0, 0, 0,        #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
##     (0.1, 1), (1, 0.0),        #alpha keys
##     (0.1, 0.6), (1, 0.6),      #red keys
##     (0.1, 0.5),(1, 0.5),       #green keys
##     (0.1, 0.4), (1, 0.4),      #blue keys
##     (0.0, 0.2),   (1, 0.8),    #scale keys
##     (0.0, 0.3, 0.0),           #emit box size
##     (0, 1, 0),                 #emit velocity
##     1,                         #emit dir randomness
##     250,                       #rotation speed
##     0,                       #rotation damping
##     ),
    
#*-*-*-**** BLOOD  END ****-*-*-*#

#-*-*-*- Fire Fly Deneme *-*-*-*-#    
     ("fire_fly_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prt_sparks_mesh_1",
     2, 5, 1.2, 0, 50, 7,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.1, 0.8), (1, 0.2),        #alpha keys
     (0.5, .7), (1, 0.7),      #red keys
     (0.5, 0.8), (1, 0.8),      #green keys
     (0.5, 1), (1, 1),      #blue keys
     (0, 0.1),   (1, 0.1),    #scale keys
     (20, 20, 0.5),             #emit box size
     (0, 0, 0),              #emit velocity
      5,                        #emit dir randomness
      0,                        #rotation speed
      0                         #rotation damping
       ),
     ("bug_fly_1", psf_billboard_2d | psf_always_emit, "prt_mesh_rose_a",
     20, 8, 0.02, 0.025, 1, 5,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 1), (1, 1),        #alpha keys
     (0, 0.5), (1, 0.5),      #red keys
     (0, 0.5), (1, 0.5),      #green keys
     (0, 0.5), (1, 0.5),      #blue keys
     (0, 0.25),   (1, 0.25),    #scale keys
     (10, 5, 0.1),             #emit box size
     (0, 0, -0.9),              #emit velocity
      0.01,                        #emit dir randomness
      10,                        #rotation speed
      0,                         #rotation damping
    ),
#-*-*-*- Fire Fly End*-*-*-*-#
#-*-*-*- Moon Beam *-*-*-*-*-*-*#
    ("moon_beam_1", psf_billboard_2d|psf_global_emit_dir|psf_always_emit|psf_randomize_size, "prt_mesh_moon_beam",#prt_mesh_moon_beam
     2, 4, 1.2, 0, 0, 0,          #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 1), (1, 0),            #alpha keys
     (0, 0.4), (1, 0.4),                #red keys
     (0, 0.5), (1, 0.5),                #green keys
     (0, 0.6), (1, 0.6),                #blue keys
     (0, 2),   (1, 2.2),        #scale keys
     (1, 1, 0.2),                 #emit box size
     (0, 0, -2),                     #emit velocity
      0,                            #emit dir randomness
      100,                          #rotation speed
      0.5,                          #rotation damping
       ),
     ("moon_beam_paricle_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size, "prt_sparks_mesh_1",
     10, 1.5, 1.5, 0, 10, 10,            #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 1), (1, 0.0),            #alpha keys
     (0.5, .5), (1, 0.5),           #red keys
     (0.5, 0.7), (1, 0.7),          #green keys
     (0.5, 1), (1, 1),              #blue keys
     (0, 0.1),   (1, 0.1),        #scale keys
     (1, 1, 4),                 #emit box size
     (0, 0, 0),                     #emit velocity
      0.5,                            #emit dir randomness
      0,                            #rotation speed
      0                             #rotation damping
       ),
#-*-*-*- Moon Beam End *-*-*-*-*-*-*#
#-*-*-*- Stone Smoke *-*-*-*-*-*-*#
##("stone_hit_1", psf_billboard_3d | psf_randomize_size | psf_randomize_rotation,  "prt_mesh_dust_1",
##     5000, 0.5, 6, 0.1, 0, 0,       #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
##     (0.1, .2), (1, 0),             #alpha keys
##     (0.5, 0.7), (1, 0.7),          #red keys
##     (0.5, 0.6), (1, 0.6),          #green keys
##     (0.5, 0.6), (1, 0.6),          #blue keys
##     (0.0, .2),   (1, 0.7),         #scale keys
##     (0, 0.3, 0),                   #emit box size
##     (0, 0, 0),                     #emit velocity
##     1.1,                           #emit dir randomness
##     200,                           #rotation speed
##     0.8,                           #rotation damping
##    ),
#-*-*-*- Stone Smoke END -*-*-*-*-*#
    ("night_smoke_1", psf_billboard_3d|psf_global_emit_dir|psf_always_emit, "prt_mesh_dust_1",
     5, 10, 1.5, 0, 50, 2,      #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.3, 0.1), (1, 0),        #alpha keys
     (0.5, 0.5), (1, 0.5),      #red keys
     (0.5, 0.5), (1, 0.5),      #green keys
     (0.5, 0.5), (1, 0.6),      #blue keys
     (0, 10),   (1, 10),        #scale keys
     (25, 25, 0.5),               #emit box size
     (0, 1, 0),                 #emit velocity
      2,                        #emit dir randomness
      20,                       #rotation speed
      1                         #rotation damping
       ),
#-*-*-*- Fire For Fireplace -*-*-*-#
    ("fireplace_fire_small", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     25, 0.8, 0.2, -0.1, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.5), (1, 0),        #alpha keys
     (0.5, 1.0), (1, 0.9),      #red keys
     (0.5, 0.7),(1, 0.3),       #green keys
     (0.5, 0.2), (1, 0.0),      #blue keys
     (0, 0.2),   (1, 0.7),   #scale keys
     (0.2, 0.1, 0.01),      #emit box size
     (0, 0, 0.2),               #emit velocity
     0.1,                       #emit dir randomness
     100,                       #rotation speed
     0.5                        #rotation damping
    ),
    ("fireplace_fire_big", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     35, 0.6, 0.2, -0.2, 10.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.5), (1, 0),        #alpha keys
     (0.5, 1.0), (1, 0.9),      #red keys
     (0.5, 0.7),(1, 0.3),       #green keys
     (0.5, 0.2), (1, 0.0),      #blue keys
     (0, 0.4),   (1, 1),   #scale keys
     (0.4, 0.2, 0.01),            #emit box size
     (0, 0, 0.4),               #emit velocity
     0.1,                       #emit dir randomness
     100,                       #rotation speed
     0.5                        #rotation damping
    ),  
#-*-*-*- Fire For Fireplace -*-*-*-#
#-*-*-*- Village Fire *-*-*-*-#
    ("village_fire_big", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     50, 1.0, 0, -1.2, 25.0, 10.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.2, 0.7), (1, 0),        #alpha keys
     (0.2, 1.0), (1, 0.9),      #red keys
     (0.2, 0.7),(1, 0.3),       #green keys
     (0.2, 0.2), (1, 0.0),      #blue keys
     (0, 2),   (1, 6),          #scale keys
     (2.2, 2.2, 0.2),           #emit box size
     (0, 0, 0.0),               #emit velocity
     0.0,                       #emit dir randomness
     250,                       #rotation speed
     0.3                        #rotation damping
    ),
    ("village_fire_smoke_big", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_smoke_1",
     30, 2, 0.3, -1, 50.0, 10.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.15), (1, 0),        #alpha keys
     (0.2, 0.4), (1, 0.2),      #red keys
     (0.2, 0.4),(1, 0.2),       #green keys
     (0.2, 0.4), (1, 0.2),      #blue keys
     (0, 6),   (1, 8),          #scale keys
     (2, 2, 1),           #emit box size
     (0, 0, 5),               #emit velocity
     0.0,                       #emit dir randomness
     0,                       #rotation speed
     0.1                        #rotation damping
    ), 
    ("map_village_fire", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_fire_1",
     20, 1.0, 0, -0.2, 3.0, 3.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.2, 0.7), (1, 0),        #alpha keys
     (0.2, 1.0), (1, 0.9),      #red keys
     (0.2, 0.7),(1, 0.3),       #green keys
     (0.2, 0.2), (1, 0.0),      #blue keys
     (0, 0.15),   (1, 0.45),    #scale keys
     (0.2, 0.2, 0.02),           #emit box size
     (0, 0, 0.0),               #emit velocity
     0.0,                       #emit dir randomness
     250,                       #rotation speed
     0.3                        #rotation damping
    ),
    ("map_village_fire_smoke", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_smoke_1",
     25, 2.5, 0.3, -0.15, 3.0, 3.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.15), (1, 0),       #alpha keys
     (0.2, 0.4), (1, 0.3),      #red keys
     (0.2, 0.4),(1, 0.3),       #green keys
     (0.2, 0.4), (1, 0.3),      #blue keys
     (0, 0.6),   (1, 0.9),      #scale keys
     (0.2, 0.2, 0.1),           #emit box size
     (0, 0, 0.03),              #emit velocity
     0.0,                       #emit dir randomness
     0,                         #rotation speed
     0.1                        #rotation damping
    ), 
    ("map_village_looted_smoke", psf_billboard_3d|psf_global_emit_dir|psf_always_emit|psf_randomize_size|psf_randomize_rotation, "prt_mesh_smoke_1",
     20, 3, 0.3, -0.11, 3.0, 2.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.5, 0.15), (1, 0),       #alpha keys
     (0.2, 0.5), (1, 0.5),      #red keys
     (0.2, 0.5),(1, 0.5),       #green keys
     (0.2, 0.5), (1, 0.5),      #blue keys
     (0, 0.7),   (1, 1.3),      #scale keys
     (0.2, 0.2, 0.1),           #emit box size
     (0, 0, 0.015),             #emit velocity
     0.0,                       #emit dir randomness
     0,                         #rotation speed
     0.1                        #rotation damping
    ), 
##### Dungeon Water Drops #####
    ("dungeon_water_drops", psf_billboard_2d|psf_global_emit_dir|psf_always_emit, "prtcl_rain",
     1, 1, 0.33, 0.8, 0, 0, #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (1.0, 0.2), (1, 0.2),      #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (1.0, 0.8),   (1.0, 0.8),  #scale keys
     (0.05, 0.05, 0.5),         #emit box size
     (0, 0, -5.0),              #emit velocity
     0.0,                       #emit dir randomness
     0,                         #rotation speed
     0,                         #rotation damping
     ), 
##### Dungeon Water Drops  END #####
    ("wedding_rose", psf_billboard_2d | psf_always_emit, "prt_mesh_rose_a",
     50, 8, 0.02, 0.025, 1, 5,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 1), (1, 1),        #alpha keys
     (0, 0.5), (1, 0.5),      #red keys
     (0, 0.5), (1, 0.5),      #green keys
     (0, 0.5), (1, 0.5),      #blue keys
     (0, 0.25),   (1, 0.25),    #scale keys
     (4, 4, 0.1),             #emit box size
     (0, 0, -0.9),              #emit velocity
      0.01,                        #emit dir randomness
      10,                        #rotation speed
      0,                         #rotation damping
    ),
    ("sea_foam_a", psf_turn_to_velocity | psf_always_emit|psf_randomize_size, "prt_foam_a",
     1, 3.0, 1, 0.0, 0.0, 0.0,     #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0.7, 0.1), (1, 0.0),        #alpha keys
     (1.0, 1.0), (1, 1.0),      #red keys
     (1.0, 1.0), (1, 1.0),      #green keys
     (1.0, 1.0), (1, 1.0),      #blue keys
     (0.0, 4),   (1.0, 4.5),   #scale keys
     (10.0, 1.0, 0),           #emit box size
     (0, 1, 0),                 #emit velocity
     0.0,                       #emit dir randomness
     0,                       #rotation speed
     0.5                        #rotation damping
    ),
    ("fall_leafs_a", psf_billboard_2d | psf_always_emit, "prt_mesh_yrellow_leaf_a",
     1, 9, 0, 0.025, 4, 4,      #num_particles, life, damping, gravity_strength, turbulance_size, turbulance_strength
     (0, 1), (1, 1),            #alpha keys
     (0, 0.5), (1, 0.5),        #red keys
     (0, 0.5), (1, 0.5),        #green keys
     (0, 0.5), (1, 0.5),        #blue keys
     (0, 0.25),   (1, 0.25),    #scale keys
     (4, 4, 4),                 #emit box size
     (0, 0.01, -0.9),           #emit velocity
      0.02,                     #emit dir randomness
      15,                       #rotation speed
      0,                        #rotation damping
    ),
]



from smart_module.module_system.module_scenes import *
from smart_module.module_system.module_troops import *
from smart_module.module_system.process_operations import *


def save_python_header(src_dir):
  ofile = open(src_dir + "ID_scenes.py","w",encoding="utf-8")
  for i_scene in range(len(scenes)):
    ofile.write("scn_%s = %d\n"%(convert_to_identifier(scenes[i_scene][0]),i_scene))
  ofile.close()





scene_name_pos = 0
passages_pos = 8
scene_outer_terrain_pos = 10


def write_vec(ofile,vec):
  ofile.write(" %f %f %f "%vec)
  
def write_passage(ofile,scenes,passage):
  scene_no = 0
  found = 0
  while (not found) and (scene_no < len(scenes)):
    if (scenes[scene_no][0] == passage):
      found = 1
    else:
      scene_no += 1
  if (passage == "exit"):
    scene_no = 100000
  elif (passage == ""):
    scene_no = 0
  elif not found:
    # print( "Error passage not found:")
    # print( passage)
    raise RuntimeError('Error passage not found:' + passage)
  ofile.write(" %d "%scene_no)


def save_scenes(variables,variable_uses,tag_uses,export_dir):
  ofile = open(export_dir + "scenes.txt","w",encoding="utf-8")
  ofile.write("scenesfile version 1\n")
  ofile.write(" %d\n"%len(scenes))
  for scene in scenes:
    ofile.write("scn_%s %s %d %s %s %f %f %f %f %f %s "%(convert_to_identifier(scene[0]),replace_spaces(scene[0]),scene[1], scene[2],scene[3],scene[4][0],scene[4][1],scene[5][0],scene[5][1],scene[6],scene[7]))
    passages = scene[passages_pos]
    ofile.write("\n  %d "%len(passages))
    for passage in passages:
      write_passage(ofile,scenes,passage)
    chest_troops = scene[9]
    ofile.write("\n  %d "%len(chest_troops))
    for chest_troop in chest_troops:
      troop_no = find_troop(troops,chest_troop)
      if (troop_no < 0):
        print( "Error unable to find chest-troop: " + chest_troop)
        troop_no = 0
      else:
        add_tag_use(tag_uses,tag_troop,troop_no)
      ofile.write(" %d "%troop_no)
    ofile.write("\n")
    if (len(scene) > scene_outer_terrain_pos):
      ofile.write(" %s "%scene[scene_outer_terrain_pos])
    else:
      ofile.write(" 0 ")
    ofile.write("\n")
  ofile.close()


def processScenes(context):
  print("Exporting scene data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_python_header(src_dir)

  variable_uses = []
  variables = load_variables(export_dir, variable_uses)
  tag_uses = load_tag_uses(export_dir)
  quick_strings = load_quick_strings(export_dir)
  save_scenes(variables,variable_uses,tag_uses,export_dir)
  save_variables(export_dir,variables,variable_uses)
  save_tag_uses(export_dir,tag_uses)
  save_quick_strings(export_dir,quick_strings)




from smart_module.module_system.module_troops import *

from smart_module.module_system.process_common import *
#from process_operations import *

num_face_numeric_keys = 4

def save_troops(export_dir):
  file = open(export_dir + "troops.txt","w",encoding="utf-8")
  file.write("troopsfile version 2\n")
  file.write("%d "%len(troops))
  for troop in troops:
    troop_len = len(troop)
    if troop_len == 11:
      troop[11:11] = [0, 0, 0, 0, 0]
    elif troop_len == 12:
      troop[12:12] = [0, 0, 0, 0]
    elif troop_len == 13:
      troop[13:13] = [0, 0, 0]
    elif troop_len == 14:
      troop[14:14] = [0, 0]
    elif troop_len == 15:
      troop[15:15] = [0]
    if (troop[4] > 0):
#      add_tag_use(tag_uses,tag_scene,troop[4] & tsf_site_id_mask)
      id_no = find_object("trp",convert_to_identifier(troop[0]))
#      if (id_no >= 0):  add_tag_use(tag_uses,tag_troop,id_no)
#    if (troop[6] > 0):  add_tag_use(tag_uses,tag_faction,troop[6])

    file.write("\ntrp_%s %s %s %s %d %d %d %d %d %d\n  "%(convert_to_identifier(troop[0]),replace_spaces(troop[1]),replace_spaces(troop[2]), replace_spaces(str(troop[13])), troop[3],troop[4],troop[5], troop[6], troop[14], troop[15]))
    inventory_list = troop[7]
#    inventory_list.append(itm_arrows)
#    inventory_list.append(itm_bolts)
    for inventory_item in inventory_list:
#      add_tag_use(tag_uses,tag_item,inventory_item)
      file.write("%d 0 "%inventory_item)
    for i in range(64 - len(inventory_list)):
      file.write("-1 0 ")
    file.write("\n ")
    attrib = troop[8]
    strength = (attrib & 0xff)
    agility  = ((attrib >> 8)& 0xff)
    intelligence = ((attrib >> 16)& 0xff)
    charisma = ((attrib >> 24)& 0xff)
    starting_level = (attrib >> level_bits) & level_mask
#    gold = two_to_pow(2 + (attrib >> 12)& 0x0f) * random
    
    file.write(" %d %d %d %d %d\n"%(strength,agility,intelligence,charisma,starting_level))
    wp_word = troop[9]
    for wp in range(num_weapon_proficiencies):
      wp_level = wp_word & 0x3FF
      file.write(" %d"%wp_level)
      wp_word = wp_word >> 10
    file.write("\n")
      
    skill_array = troop[10]
    for i in range(num_skill_words):
      file.write("%d "%((skill_array >> (i * 32)) & 0xffffffff))
    file.write("\n  ")

    face_keys = [troop[11],troop[12]]
    
    for fckey in (face_keys):
      word_keys = []
      for word_no in range(num_face_numeric_keys):
        word_keys.append((fckey >> (64 * word_no)) & 0xFFFFFFFFFFFFFFFF)
      for word_no in range(num_face_numeric_keys):
        file.write("%d "%(word_keys[(num_face_numeric_keys -1) - word_no]))

    file.write("\n")
      
        

 #     word2 = (fckey >> 64) & 0xFFFFFFFFFFFFFFFF
 #     word3 = (fckey >> 128) & 0xFFFFFFFFFFFFFFFF
 #     word4 = (fckey >> 192) & 0xFFFFFFFFFFFFFFFF
#      file.write("%d %d %d %d "%(word4, word3, word2, word1))

#    face_keys = troop[10]
#    for fckey in (face_keys):
#      file.write("%d "%(fckey))
#    for i in range(4 - len(face_keys)):
#      file.write("0 ")
      
    
  file.close()

def two_to_pow(x):
  result = 1
  for i in range(x):
    result = result * 2
  return result

def save_python_header(src_dir):
  file = open(src_dir + "ID_troops.py","w",encoding="utf-8")
  for i_troop in range(len(troops)):
    file.write("trp_%s = %d\n"%(convert_to_identifier(troops[i_troop][0]),i_troop))
  file.close()



def processTroops(context):
    print( "Exporting troops data")

    configParser = context.configParser
    src_dir = configParser.getSrcDir()
    export_dir = configParser.getExportDir()

    #tag_uses = load_tag_uses(export_dir)
    save_python_header(src_dir)
    save_troops(export_dir)
    #save_tag_uses(export_dir, tag_uses)
    #print( "Generating C header...")
    #save_c_header()
    #print( "Generating Python header...")
    #print( "Finished.")



# -*- coding: utf-8 -*-
from smart_module.module_system.module_scripts import *

'''
    繁荣度功能（星期为单位）
    1.基础增加2点繁荣
    2.有庄园再增加3点繁荣
    2.有磨坊再增加5点繁荣
    2.有了望塔再增加1点繁荣
    2.有学校再增加2点繁荣
    2.有驿站再增加1点繁荣
    2.有监狱再增加1点繁荣
    4.城镇拥有工厂时增加7点繁荣
    
    
    被洗劫后不增加繁荣

'''


## 繁荣度多久更新一次（24*7）（游戏中的单位：小时）
prosperity_update_interval = 24 * 7

## 每一次更新必然增加的繁荣度
center_base_prosperity = 2

'''
    庄园繁荣度加成
'''
center_has_manor_prosperity= 3 #village
'''
    磨坊繁荣度加成
'''
center_has_mill_prosperity= 5 #village
'''
    了望塔繁荣度加成
'''
center_has_watch_tower_prosperity= 1 #village
'''
    学校繁荣度加成
'''
center_has_school_prosperity= 2 #village
'''
    驿站繁荣度加成
'''
center_has_messenger_post_prosperity= 1 #town, castle, village
'''
    监狱繁荣度加成
'''
center_has_prisoner_tower_prosperity= 1 #town, castle

'''
    工厂繁荣度加成
'''
center_player_enterprise_prosperity = 7


centerEnhanceProsperity = {
    "name":"CenterEnhanceProsperity",
    "enable":True,
    "dependentOn":["PartyBaseScripts"],
    "version": "v2.0.0",
    "desc":'''
        繁荣度功能（星期为单位）
        1.基础增加2点繁荣
        2.有庄园再增加3点繁荣
        2.有磨坊再增加5点繁荣
        2.有了望塔再增加1点繁荣
        2.有学校再增加2点繁荣
        2.有驿站再增加1点繁荣
        2.有监狱再增加1点繁荣
        4.城镇拥有工厂时增加7点繁荣
        
        被洗劫后不增加繁荣
    ''',
    "actions": [
        ("Append@simple_triggers",[
            (prosperity_update_interval, [
                # (display_message,"@prisperity init"),
                (call_script, "script_update_prisperity_for_all_center"),
            ]),
        ]),
        ("Append@scripts",[
            ## 巡逻队的核心功能，驱动巡逻队运行的代码
            ("update_prisperity_for_all_center", [
                ## 领地巡逻队
                (try_for_range, ":center_no", centers_begin, centers_end),
                    (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_town),
                    (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_castle),
                    (party_slot_eq, ":center_no", slot_party_type, spt_village),

                    ## 城堡没有被围攻
                    (this_or_next | party_slot_eq, ":center_no", slot_center_is_besieged_by, -1),
                    ## 村庄是正常状态
                    (party_slot_eq, ":center_no", slot_village_state, svs_normal),

                    (assign, ":prosperity", center_base_prosperity),
                    ## 庄园加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_has_manor, 1),
                    (val_add, ":prosperity", center_has_manor_prosperity),
                    (try_end),
                    ## 磨坊加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_has_fish_pond, 1),
                    (val_add, ":prosperity", center_has_mill_prosperity),
                    (try_end),
                    ## 了望塔加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_has_watch_tower, 1),
                    (val_add, ":prosperity", center_has_watch_tower_prosperity),
                    (try_end),
                    ## 学校加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_has_school, 1),
                    (val_add, ":prosperity", center_has_school_prosperity),
                    (try_end),
                    ## 驿站加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_has_messenger_post, 1),
                    (val_add, ":prosperity", center_has_messenger_post_prosperity),
                    (try_end),
                    ## 监狱加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_has_prisoner_tower, 1),
                    (val_add, ":prosperity", center_has_prisoner_tower_prosperity),
                    (try_end),
                    ## 玩家工厂加成
                    (try_begin),
                    (party_slot_eq, ":center_no", slot_center_player_enterprise, 1),
                    (val_add, ":prosperity", center_player_enterprise_prosperity),
                    (try_end),

                    (party_get_slot, ":center_prosperity", ":center_no", slot_town_prosperity),
                    (val_add, ":center_prosperity", ":prosperity"),
                    (party_set_slot, ":center_no", slot_town_prosperity, ":center_prosperity"),

                    (assign,reg1,":prosperity"),
                    (str_store_party_name_link,s1,":center_no"),
                    (display_message,"@{s1} 增 加 {reg1} 点 繁 荣 度"),
                (try_end),
            ]),
        ]),
    ],
}


itm_no_item = 0
itm_tutorial_spear = 1
itm_tutorial_club = 2
itm_tutorial_battle_axe = 3
itm_tutorial_arrows = 4
itm_tutorial_bolts = 5
itm_tutorial_short_bow = 6
itm_tutorial_crossbow = 7
itm_tutorial_throwing_daggers = 8
itm_tutorial_saddle_horse = 9
itm_tutorial_shield = 10
itm_tutorial_staff_no_attack = 11
itm_tutorial_staff = 12
itm_tutorial_sword = 13
itm_tutorial_axe = 14
itm_tutorial_dagger = 15
itm_horse_meat = 16
itm_practice_sword = 17
itm_heavy_practice_sword = 18
itm_practice_dagger = 19
itm_practice_axe = 20
itm_arena_axe = 21
itm_arena_sword = 22
itm_arena_sword_two_handed = 23
itm_arena_lance = 24
itm_practice_staff = 25
itm_practice_lance = 26
itm_practice_shield = 27
itm_practice_bow = 28
itm_practice_crossbow = 29
itm_practice_javelin = 30
itm_practice_javelin_melee = 31
itm_practice_throwing_daggers = 32
itm_practice_throwing_daggers_100_amount = 33
itm_practice_horse = 34
itm_practice_arrows = 35
itm_practice_bolts = 36
itm_practice_arrows_10_amount = 37
itm_practice_arrows_100_amount = 38
itm_practice_bolts_9_amount = 39
itm_practice_boots = 40
itm_red_tourney_armor = 41
itm_blue_tourney_armor = 42
itm_green_tourney_armor = 43
itm_gold_tourney_armor = 44
itm_red_tourney_helmet = 45
itm_blue_tourney_helmet = 46
itm_green_tourney_helmet = 47
itm_gold_tourney_helmet = 48
itm_arena_shield_red = 49
itm_arena_shield_blue = 50
itm_arena_shield_green = 51
itm_arena_shield_yellow = 52
itm_arena_armor_white = 53
itm_arena_armor_red = 54
itm_arena_armor_blue = 55
itm_arena_armor_green = 56
itm_arena_armor_yellow = 57
itm_arena_tunic_white = 58
itm_arena_tunic_red = 59
itm_arena_tunic_blue = 60
itm_arena_tunic_green = 61
itm_arena_tunic_yellow = 62
itm_arena_helmet_red = 63
itm_arena_helmet_blue = 64
itm_arena_helmet_green = 65
itm_arena_helmet_yellow = 66
itm_steppe_helmet_white = 67
itm_steppe_helmet_red = 68
itm_steppe_helmet_blue = 69
itm_steppe_helmet_green = 70
itm_steppe_helmet_yellow = 71
itm_tourney_helm_white = 72
itm_tourney_helm_red = 73
itm_tourney_helm_blue = 74
itm_tourney_helm_green = 75
itm_tourney_helm_yellow = 76
itm_arena_turban_red = 77
itm_arena_turban_blue = 78
itm_arena_turban_green = 79
itm_arena_turban_yellow = 80
itm_book_tactics = 81
itm_book_persuasion = 82
itm_book_leadership = 83
itm_book_intelligence = 84
itm_book_trade = 85
itm_book_weapon_mastery = 86
itm_book_engineering = 87
itm_book_wound_treatment_reference = 88
itm_book_training_reference = 89
itm_book_surgery_reference = 90
itm_spice = 91
itm_salt = 92
itm_oil = 93
itm_pottery = 94
itm_raw_flax = 95
itm_linen = 96
itm_wool = 97
itm_wool_cloth = 98
itm_raw_silk = 99
itm_raw_dyes = 100
itm_velvet = 101
itm_iron = 102
itm_tools = 103
itm_raw_leather = 104
itm_leatherwork = 105
itm_raw_date_fruit = 106
itm_furs = 107
itm_wine = 108
itm_ale = 109
itm_smoked_fish = 110
itm_cheese = 111
itm_honey = 112
itm_sausages = 113
itm_cabbages = 114
itm_dried_meat = 115
itm_apples = 116
itm_raw_grapes = 117
itm_raw_olives = 118
itm_grain = 119
itm_cattle_meat = 120
itm_bread = 121
itm_chicken = 122
itm_pork = 123
itm_butter = 124
itm_siege_supply = 125
itm_quest_wine = 126
itm_quest_ale = 127
itm_tutorial_sword = 128
itm_tutorial_axe = 129
itm_tutorial_spear = 130
itm_tutorial_club = 131
itm_tutorial_battle_axe = 132
itm_tutorial_arrows = 133
itm_tutorial_bolts = 134
itm_tutorial_short_bow = 135
itm_tutorial_crossbow = 136
itm_tutorial_throwing_daggers = 137
itm_tutorial_saddle_horse = 138
itm_tutorial_shield = 139
itm_tutorial_staff_no_attack = 140
itm_tutorial_staff = 141
itm_sumpter_horse = 142
itm_saddle_horse = 143
itm_steppe_horse = 144
itm_arabian_horse_a = 145
itm_courser = 146
itm_arabian_horse_b = 147
itm_hunter = 148
itm_warhorse = 149
itm_charger = 150
itm_arrows = 151
itm_khergit_arrows = 152
itm_barbed_arrows = 153
itm_bodkin_arrows = 154
itm_bolts = 155
itm_steel_bolts = 156
itm_cartridges = 157
itm_pilgrim_disguise = 158
itm_pilgrim_hood = 159
itm_leather_gloves = 160
itm_mail_mittens = 161
itm_scale_gauntlets = 162
itm_lamellar_gauntlets = 163
itm_gauntlets = 164
itm_wrapping_boots = 165
itm_woolen_hose = 166
itm_blue_hose = 167
itm_hunter_boots = 168
itm_hide_boots = 169
itm_ankle_boots = 170
itm_nomad_boots = 171
itm_leather_boots = 172
itm_splinted_leather_greaves = 173
itm_mail_chausses = 174
itm_splinted_greaves = 175
itm_mail_boots = 176
itm_iron_greaves = 177
itm_black_greaves = 178
itm_khergit_leather_boots = 179
itm_sarranid_boots_a = 180
itm_sarranid_boots_b = 181
itm_sarranid_boots_c = 182
itm_sarranid_boots_d = 183
itm_sarranid_head_cloth = 184
itm_sarranid_head_cloth_b = 185
itm_sarranid_felt_head_cloth = 186
itm_sarranid_felt_head_cloth_b = 187
itm_lady_dress_ruby = 188
itm_lady_dress_green = 189
itm_lady_dress_blue = 190
itm_red_dress = 191
itm_brown_dress = 192
itm_green_dress = 193
itm_khergit_lady_dress = 194
itm_khergit_lady_dress_b = 195
itm_sarranid_lady_dress = 196
itm_sarranid_lady_dress_b = 197
itm_sarranid_common_dress = 198
itm_sarranid_common_dress_b = 199
itm_courtly_outfit = 200
itm_nobleman_outfit = 201
itm_nomad_armor = 202
itm_khergit_armor = 203
itm_leather_jacket = 204
itm_rawhide_coat = 205
itm_leather_armor = 206
itm_fur_coat = 207
itm_coat = 208
itm_leather_coat = 209
itm_mail_coat = 210
itm_long_mail_coat = 211
itm_mail_with_tunic_red = 212
itm_mail_with_tunic_green = 213
itm_hide_coat = 214
itm_merchant_outfit = 215
itm_homespun_dress = 216
itm_thick_coat = 217
itm_coat_with_cape = 218
itm_steppe_outfit = 219
itm_nordic_outfit = 220
itm_nordic_armor = 221
itm_hide_armor = 222
itm_cloaked_tunic = 223
itm_sleeveless_tunic = 224
itm_sleeveless_leather_tunic = 225
itm_linen_shirt = 226
itm_wool_coat = 227
itm_dress = 228
itm_blue_dress = 229
itm_peasant_dress = 230
itm_woolen_dress = 231
itm_shirt = 232
itm_linen_tunic = 233
itm_short_tunic = 234
itm_red_shirt = 235
itm_red_tunic = 236
itm_green_tunic = 237
itm_blue_tunic = 238
itm_robe = 239
itm_coarse_tunic = 240
itm_leather_apron = 241
itm_tabard = 242
itm_leather_vest = 243
itm_steppe_armor = 244
itm_gambeson = 245
itm_blue_gambeson = 246
itm_red_gambeson = 247
itm_padded_cloth = 248
itm_aketon_green = 249
itm_leather_jerkin = 250
itm_nomad_vest = 251
itm_ragged_outfit = 252
itm_padded_leather = 253
itm_tribal_warrior_outfit = 254
itm_nomad_robe = 255
itm_studded_leather_coat = 256
itm_byrnie = 257
itm_haubergeon = 258
itm_lamellar_vest = 259
itm_lamellar_vest_khergit = 260
itm_mail_shirt = 261
itm_mail_hauberk = 262
itm_mail_with_surcoat = 263
itm_surcoat_over_mail = 264
itm_brigandine_red = 265
itm_lamellar_armor = 266
itm_scale_armor = 267
itm_banded_armor = 268
itm_cuir_bouilli = 269
itm_coat_of_plates = 270
itm_coat_of_plates_red = 271
itm_plate_armor = 272
itm_black_armor = 273
itm_pelt_coat = 274
itm_khergit_elite_armor = 275
itm_vaegir_elite_armor = 276
itm_sarranid_elite_armor = 277
itm_sarranid_dress_a = 278
itm_sarranid_dress_b = 279
itm_sarranid_cloth_robe = 280
itm_sarranid_cloth_robe_b = 281
itm_skirmisher_armor = 282
itm_archers_vest = 283
itm_sarranid_leather_armor = 284
itm_sarranid_cavalry_robe = 285
itm_arabian_armor_b = 286
itm_sarranid_mail_shirt = 287
itm_mamluke_mail = 288
itm_burlap_tunic = 289
itm_heraldic_mail_with_surcoat = 290
itm_heraldic_mail_with_tunic = 291
itm_heraldic_mail_with_tunic_b = 292
itm_heraldic_mail_with_tabard = 293
itm_turret_hat_ruby = 294
itm_turret_hat_blue = 295
itm_turret_hat_green = 296
itm_head_wrappings = 297
itm_court_hat = 298
itm_wimple_a = 299
itm_wimple_with_veil = 300
itm_straw_hat = 301
itm_common_hood = 302
itm_hood_b = 303
itm_hood_c = 304
itm_hood_d = 305
itm_headcloth = 306
itm_woolen_hood = 307
itm_arming_cap = 308
itm_fur_hat = 309
itm_nomad_cap = 310
itm_nomad_cap_b = 311
itm_steppe_cap = 312
itm_padded_coif = 313
itm_woolen_cap = 314
itm_felt_hat = 315
itm_felt_hat_b = 316
itm_leather_cap = 317
itm_female_hood = 318
itm_leather_steppe_cap_a = 319
itm_leather_steppe_cap_b = 320
itm_leather_steppe_cap_c = 321
itm_leather_warrior_cap = 322
itm_skullcap = 323
itm_mail_coif = 324
itm_footman_helmet = 325
itm_nasal_helmet = 326
itm_norman_helmet = 327
itm_segmented_helmet = 328
itm_helmet_with_neckguard = 329
itm_flat_topped_helmet = 330
itm_kettle_hat = 331
itm_spiked_helmet = 332
itm_nordic_helmet = 333
itm_khergit_lady_hat = 334
itm_khergit_lady_hat_b = 335
itm_sarranid_felt_hat = 336
itm_turban = 337
itm_desert_turban = 338
itm_sarranid_warrior_cap = 339
itm_sarranid_horseman_helmet = 340
itm_sarranid_helmet1 = 341
itm_sarranid_mail_coif = 342
itm_sarranid_veiled_helmet = 343
itm_nordic_archer_helmet = 344
itm_nordic_veteran_archer_helmet = 345
itm_nordic_footman_helmet = 346
itm_nordic_fighter_helmet = 347
itm_nordic_huscarl_helmet = 348
itm_nordic_warlord_helmet = 349
itm_vaegir_fur_cap = 350
itm_vaegir_fur_helmet = 351
itm_vaegir_spiked_helmet = 352
itm_vaegir_lamellar_helmet = 353
itm_vaegir_noble_helmet = 354
itm_vaegir_war_helmet = 355
itm_vaegir_mask = 356
itm_bascinet = 357
itm_bascinet_2 = 358
itm_bascinet_3 = 359
itm_guard_helmet = 360
itm_black_helmet = 361
itm_full_helm = 362
itm_great_helmet = 363
itm_winged_great_helmet = 364
itm_wooden_stick = 365
itm_cudgel = 366
itm_hammer = 367
itm_club = 368
itm_winged_mace = 369
itm_spiked_mace = 370
itm_military_hammer = 371
itm_maul = 372
itm_sledgehammer = 373
itm_warhammer = 374
itm_pickaxe = 375
itm_spiked_club = 376
itm_fighting_pick = 377
itm_military_pick = 378
itm_morningstar = 379
itm_sickle = 380
itm_cleaver = 381
itm_knife = 382
itm_butchering_knife = 383
itm_dagger = 384
itm_falchion = 385
itm_scimitar = 386
itm_scimitar_b = 387
itm_arabian_sword_a = 388
itm_arabian_sword_b = 389
itm_sarranid_cavalry_sword = 390
itm_arabian_sword_d = 391
itm_great_sword = 392
itm_sword_of_war = 393
itm_hatchet = 394
itm_hand_axe = 395
itm_fighting_axe = 396
itm_axe = 397
itm_voulge = 398
itm_battle_axe = 399
itm_war_axe = 400
itm_sword_two_handed_b = 401
itm_sword_two_handed_a = 402
itm_khergit_sword_two_handed_a = 403
itm_khergit_sword_two_handed_b = 404
itm_two_handed_cleaver = 405
itm_military_cleaver_b = 406
itm_military_cleaver_c = 407
itm_military_sickle_a = 408
itm_bastard_sword_a = 409
itm_bastard_sword_b = 410
itm_one_handed_war_axe_a = 411
itm_one_handed_battle_axe_a = 412
itm_one_handed_war_axe_b = 413
itm_one_handed_battle_axe_b = 414
itm_one_handed_battle_axe_c = 415
itm_two_handed_axe = 416
itm_two_handed_battle_axe_2 = 417
itm_shortened_voulge = 418
itm_great_axe = 419
itm_long_axe = 420
itm_long_axe_alt = 421
itm_long_axe_b = 422
itm_long_axe_b_alt = 423
itm_long_axe_c = 424
itm_long_axe_c_alt = 425
itm_bardiche = 426
itm_great_bardiche = 427
itm_voulge = 428
itm_long_bardiche = 429
itm_great_long_bardiche = 430
itm_hafted_blade_b = 431
itm_hafted_blade_a = 432
itm_shortened_military_scythe = 433
itm_sword_medieval_a = 434
itm_sword_medieval_b = 435
itm_sword_medieval_b_small = 436
itm_sword_medieval_c = 437
itm_sword_medieval_c_small = 438
itm_sword_medieval_c_long = 439
itm_sword_medieval_d_long = 440
itm_sword_viking_1 = 441
itm_sword_viking_2 = 442
itm_sword_viking_2_small = 443
itm_sword_viking_3 = 444
itm_sword_viking_3_small = 445
itm_sword_khergit_1 = 446
itm_sword_khergit_2 = 447
itm_sword_khergit_3 = 448
itm_sword_khergit_4 = 449
itm_mace_1 = 450
itm_mace_2 = 451
itm_mace_3 = 452
itm_mace_4 = 453
itm_club_with_spike_head = 454
itm_long_spiked_club = 455
itm_long_hafted_knobbed_mace = 456
itm_long_hafted_spiked_mace = 457
itm_sarranid_two_handed_mace_1 = 458
itm_sarranid_mace_1 = 459
itm_sarranid_axe_a = 460
itm_sarranid_axe_b = 461
itm_sarranid_two_handed_axe_a = 462
itm_sarranid_two_handed_axe_b = 463
itm_scythe = 464
itm_pitch_fork = 465
itm_military_fork = 466
itm_battle_fork = 467
itm_boar_spear = 468
itm_jousting_lance = 469
itm_double_sided_lance = 470
itm_glaive = 471
itm_poleaxe = 472
itm_polehammer = 473
itm_staff = 474
itm_quarter_staff = 475
itm_iron_staff = 476
itm_shortened_spear = 477
itm_spear = 478
itm_bamboo_spear = 479
itm_war_spear = 480
itm_military_scythe = 481
itm_light_lance = 482
itm_lance = 483
itm_heavy_lance = 484
itm_great_lance = 485
itm_pike = 486
itm_ashwood_pike = 487
itm_awlpike = 488
itm_awlpike_long = 489
itm_bec_de_corbin_a = 490
itm_wooden_shield = 491
itm_nordic_shield = 492
itm_fur_covered_shield = 493
itm_steel_shield = 494
itm_plate_covered_round_shield = 495
itm_leather_covered_round_shield = 496
itm_hide_covered_round_shield = 497
itm_shield_heater_c = 498
itm_norman_shield_1 = 499
itm_norman_shield_2 = 500
itm_norman_shield_3 = 501
itm_norman_shield_4 = 502
itm_norman_shield_5 = 503
itm_norman_shield_6 = 504
itm_norman_shield_7 = 505
itm_norman_shield_8 = 506
itm_tab_shield_round_a = 507
itm_tab_shield_round_b = 508
itm_tab_shield_round_c = 509
itm_tab_shield_round_d = 510
itm_tab_shield_round_e = 511
itm_tab_shield_kite_a = 512
itm_tab_shield_kite_b = 513
itm_tab_shield_kite_c = 514
itm_tab_shield_kite_d = 515
itm_tab_shield_kite_cav_a = 516
itm_tab_shield_kite_cav_b = 517
itm_tab_shield_heater_a = 518
itm_tab_shield_heater_b = 519
itm_tab_shield_heater_c = 520
itm_tab_shield_heater_d = 521
itm_tab_shield_heater_cav_a = 522
itm_tab_shield_heater_cav_b = 523
itm_tab_shield_pavise_a = 524
itm_tab_shield_pavise_b = 525
itm_tab_shield_pavise_c = 526
itm_tab_shield_pavise_d = 527
itm_tab_shield_small_round_a = 528
itm_tab_shield_small_round_b = 529
itm_tab_shield_small_round_c = 530
itm_darts = 531
itm_war_darts = 532
itm_javelin = 533
itm_javelin_melee = 534
itm_throwing_spears = 535
itm_throwing_spear_melee = 536
itm_jarid = 537
itm_jarid_melee = 538
itm_stones = 539
itm_throwing_knives = 540
itm_throwing_daggers = 541
itm_light_throwing_axes = 542
itm_light_throwing_axes_melee = 543
itm_throwing_axes = 544
itm_throwing_axes_melee = 545
itm_heavy_throwing_axes = 546
itm_heavy_throwing_axes_melee = 547
itm_hunting_bow = 548
itm_short_bow = 549
itm_nomad_bow = 550
itm_long_bow = 551
itm_khergit_bow = 552
itm_strong_bow = 553
itm_war_bow = 554
itm_hunting_crossbow = 555
itm_light_crossbow = 556
itm_crossbow = 557
itm_heavy_crossbow = 558
itm_sniper_crossbow = 559
itm_flintlock_pistol = 560
itm_torch = 561
itm_lyre = 562
itm_lute = 563
itm_strange_armor = 564
itm_strange_boots = 565
itm_strange_helmet = 566
itm_strange_sword = 567
itm_strange_great_sword = 568
itm_strange_short_sword = 569
itm_court_dress = 570
itm_rich_outfit = 571
itm_khergit_guard_armor = 572
itm_felt_steppe_cap = 573
itm_khergit_war_helmet = 574
itm_khergit_helmet = 575
itm_khergit_guard_boots = 576
itm_khergit_guard_helmet = 577
itm_khergit_cavalry_helmet = 578
itm_black_hood = 579
itm_light_leather = 580
itm_light_leather_boots = 581
itm_mail_and_plate = 582
itm_light_mail_and_plate = 583
itm_byzantion_helmet_a = 584
itm_magyar_helmet_a = 585
itm_rus_helmet_a = 586
itm_sipahi_helmet_a = 587
itm_shahi = 588
itm_rabati = 589
itm_tunic_with_green_cape = 590
itm_keys = 591
itm_bride_dress = 592
itm_bride_crown = 593
itm_bride_shoes = 594
itm_practice_bow_2 = 595
itm_practice_arrows_2 = 596
itm_plate_boots = 597
itm_heraldic_mail_with_surcoat_for_tableau = 598
itm_mail_boots_for_tableau = 599
itm_warhorse_sarranid = 600
itm_warhorse_steppe = 601
itm_items_end = 602



###################################################
# header_map_icons.py
# This file contains declarations for module_map_icons
# DO NOT EDIT THIS FILE!
###################################################

from smart_module.module_system.header_common import *

## map icon flags

mcn_no_shadow      = 0x00000001




from smart_module.module_system.module_scripts import *

'''
    建筑功能
    游戏开始时，为所有据点随机生成建筑

'''

'''
    庄园建造机率
'''
center_has_manor_possibility= 30 #village
'''
    磨坊建造机率
'''
center_has_mill_possibility= 90 #village
'''
    了望塔建造机率
'''
center_has_watch_tower_possibility= 80 #village
'''
    学校繁建造机率
'''
center_has_school_possibility= 10 #village
'''
    驿站建造机率
'''
center_has_messenger_post_possibility= 80 #town, castle, village
'''
    监狱建造机率
'''
center_has_prisoner_tower_possibility= 20 #town, castle


initCenterBuilding = {
    "name":"InitCenterBuilding",
    "enable":True,
    "version":"v2.0.0",
    "desc":'''
        建筑功能
        游戏开始时，为所有据点随机生成建筑
    ''',
    "actions":[
        ("GameInitScript",[
            ## 游戏开始时执行
            (call_script, "script_init_building_when_game_start"),
        ]),
        ("Append@scripts",[
            ("init_building_when_game_start", [
                (try_for_range, ":center_no", centers_begin, centers_end),
                    (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_town),
                    (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_castle),
                    (party_slot_eq, ":center_no", slot_party_type, spt_village),

                    # (str_store_party_name_link,s1,":center_no"),

                    ## 庄园建造
                    (store_random_in_range, ":build_possibility", 1, 101),
                    (try_begin),
                    (le, ":build_possibility", center_has_manor_possibility),
                    (party_set_slot, ":center_no", slot_center_has_manor, 1),
                    # (display_message,"@{s1} 建 造 庄 园"),
                    (try_end),
                    ## 磨坊建造
                    (store_random_in_range, ":build_possibility", 1, 101),
                    (try_begin),
                    (le, ":build_possibility", center_has_mill_possibility),
                    (party_set_slot, ":center_no", slot_center_has_fish_pond, 1),
                    # (display_message,"@{s1} 建 造 磨 坊"),
                    (try_end),
                    ## 了望塔建造
                    (store_random_in_range, ":build_possibility", 1, 101),
                    (try_begin),
                    (le, ":build_possibility", center_has_watch_tower_possibility),
                    (party_set_slot, ":center_no", slot_center_has_watch_tower, 1),
                    # (display_message,"@{s1} 建 造 了 望 塔"),
                    (try_end),
                    ## 学校建造
                    (store_random_in_range, ":build_possibility", 1, 101),
                    (try_begin),
                    (le, ":build_possibility", center_has_school_possibility),
                    (party_set_slot, ":center_no", slot_center_has_school, 1),
                    # (display_message,"@{s1} 建 造 学 校"),
                    (try_end),
                    ## 驿站建造
                    (store_random_in_range, ":build_possibility", 1, 101),
                    (try_begin),
                    (le, ":build_possibility", center_has_messenger_post_possibility),
                    (party_set_slot, ":center_no", slot_center_has_messenger_post, 1),
                    # (display_message,"@{s1} 建 造 驿 站"),
                    (try_end),
                    ## 监狱建造
                    (store_random_in_range, ":build_possibility", 1, 101),
                    (try_begin),
                    (le, ":build_possibility", center_has_prisoner_tower_possibility),
                    (party_set_slot, ":center_no", slot_center_has_prisoner_tower, 1),
                    # (display_message,"@{s1} 建 造 监 狱"),
                    (try_end),
                (try_end),
            ]),
        ]),
    ],
}



from smart_module.module_system.module_dialogs import *
from smart_module.module_system.module_game_menus import game_menus
from smart_module.module_system.module_info import *
from smart_module.module_system.module_mission_templates import mission_templates
from smart_module.module_system.module_presentations import *
from smart_module.module_system.module_scene_props import scene_props
from smart_module.module_system.module_scripts import scripts
from smart_module.module_system.module_simple_triggers import *
from smart_module.module_system.module_triggers import *
from smart_module.module_system.module_variables import reserved_variables
from smart_module.module_system.process_operations import *

def compile_all_global_vars(variable_list,variable_uses, triggers, sentences, game_menus, mission_templates, scripts, simple_triggers):
  temp_list = []
  list_type = type(temp_list)
  for varb in reserved_variables:
    try:
      add_variable(varb, variable_list, variable_uses)
    except:
      print( "ERROR in variable:")
      ##print( variable)
  
  for trigger in triggers:
    try:
      compile_global_vars(trigger[3], variable_list,variable_uses),
      compile_global_vars(trigger[4], variable_list,variable_uses),
    except:
      print( "ERROR in trigger:")
      print( trigger)

  for scene_prop in scene_props:
    try:
      sp_triggers = scene_prop[4]
      for sp_trigger in sp_triggers:
        compile_global_vars(sp_trigger[1], variable_list,variable_uses)
    except:
      print( "ERROR in scene prop:")
      print( scene_prop)
      
  for sentence in sentences:
    try:
      compile_global_vars(sentence[2], variable_list,variable_uses),
      compile_global_vars(sentence[5], variable_list,variable_uses),
    except:
      print( "ERROR in dialog line:")
      print( sentence)

  for game_menu in game_menus:
    try:
      compile_global_vars(game_menu[4], variable_list,variable_uses)
      menu_items = game_menu[5]
      for menu_item in menu_items:
        compile_global_vars(menu_item[1], variable_list,variable_uses)
        compile_global_vars(menu_item[3], variable_list,variable_uses)
    except:
      print( "ERROR in game menu:")
      print( game_menu)

  for mission_template in mission_templates:
    try:
      mt_triggers = mission_template[5]
      for mt_trigger in mt_triggers:
        compile_global_vars(mt_trigger[3], variable_list,variable_uses)
        compile_global_vars(mt_trigger[4], variable_list,variable_uses)
    except:
      print( "ERROR in mission template:")
      print( mission_template)

  for presentation in presentations:
    try:
      prsnt_triggers = presentation[3]
      for prsnt_trigger in prsnt_triggers:
        compile_global_vars(prsnt_trigger[1], variable_list,variable_uses)
    except RuntimeError as ex:
      print( "ERROR in presentation:")
      print( presentation)

  for i_script in range(len(scripts)):
    try:
      func = scripts[i_script]
      if (type(func[1]) == list_type):
        compile_global_vars(func[1], variable_list,variable_uses)
      else:
        compile_global_vars(func[2], variable_list,variable_uses)
    except RuntimeError as ex:
      print( "ERROR in script:")
      print( func)

  for simple_trigger in simple_triggers:
    try:
      compile_global_vars(simple_trigger[1]  , variable_list,variable_uses)
    except:
      print( "ERROR in simple trigger:")
      print( simple_trigger)



def processGlobalVariables(context):
  print( "Compiling all global variables...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  variable_uses = []
  variables = load_variables(export_dir, variable_uses)


  compile_all_global_vars(variables, variable_uses,triggers, dialogs, game_menus, mission_templates, scripts, simple_triggers)
  save_variables(export_dir, variables,variable_uses)



snd_click = 0
snd_tutorial_1 = 1
snd_tutorial_2 = 2
snd_gong = 3
snd_quest_taken = 4
snd_quest_completed = 5
snd_quest_succeeded = 6
snd_quest_concluded = 7
snd_quest_failed = 8
snd_quest_cancelled = 9
snd_rain = 10
snd_money_received = 11
snd_money_paid = 12
snd_sword_clash_1 = 13
snd_sword_clash_2 = 14
snd_sword_clash_3 = 15
snd_sword_swing = 16
snd_footstep_grass = 17
snd_footstep_wood = 18
snd_footstep_water = 19
snd_footstep_horse = 20
snd_footstep_horse_1b = 21
snd_footstep_horse_1f = 22
snd_footstep_horse_2b = 23
snd_footstep_horse_2f = 24
snd_footstep_horse_3b = 25
snd_footstep_horse_3f = 26
snd_footstep_horse_4b = 27
snd_footstep_horse_4f = 28
snd_footstep_horse_5b = 29
snd_footstep_horse_5f = 30
snd_jump_begin = 31
snd_jump_end = 32
snd_jump_begin_water = 33
snd_jump_end_water = 34
snd_horse_jump_begin = 35
snd_horse_jump_end = 36
snd_horse_jump_begin_water = 37
snd_horse_jump_end_water = 38
snd_release_bow = 39
snd_release_crossbow = 40
snd_throw_javelin = 41
snd_throw_axe = 42
snd_throw_knife = 43
snd_throw_stone = 44
snd_reload_crossbow = 45
snd_reload_crossbow_continue = 46
snd_pull_bow = 47
snd_pull_arrow = 48
snd_arrow_pass_by = 49
snd_bolt_pass_by = 50
snd_javelin_pass_by = 51
snd_stone_pass_by = 52
snd_axe_pass_by = 53
snd_knife_pass_by = 54
snd_bullet_pass_by = 55
snd_incoming_arrow_hit_ground = 56
snd_incoming_bolt_hit_ground = 57
snd_incoming_javelin_hit_ground = 58
snd_incoming_stone_hit_ground = 59
snd_incoming_axe_hit_ground = 60
snd_incoming_knife_hit_ground = 61
snd_incoming_bullet_hit_ground = 62
snd_outgoing_arrow_hit_ground = 63
snd_outgoing_bolt_hit_ground = 64
snd_outgoing_javelin_hit_ground = 65
snd_outgoing_stone_hit_ground = 66
snd_outgoing_axe_hit_ground = 67
snd_outgoing_knife_hit_ground = 68
snd_outgoing_bullet_hit_ground = 69
snd_draw_sword = 70
snd_put_back_sword = 71
snd_draw_greatsword = 72
snd_put_back_greatsword = 73
snd_draw_axe = 74
snd_put_back_axe = 75
snd_draw_greataxe = 76
snd_put_back_greataxe = 77
snd_draw_spear = 78
snd_put_back_spear = 79
snd_draw_crossbow = 80
snd_put_back_crossbow = 81
snd_draw_revolver = 82
snd_put_back_revolver = 83
snd_draw_dagger = 84
snd_put_back_dagger = 85
snd_draw_bow = 86
snd_put_back_bow = 87
snd_draw_shield = 88
snd_put_back_shield = 89
snd_draw_other = 90
snd_put_back_other = 91
snd_body_fall_small = 92
snd_body_fall_big = 93
snd_horse_body_fall_begin = 94
snd_horse_body_fall_end = 95
snd_hit_wood_wood = 96
snd_hit_metal_metal = 97
snd_hit_wood_metal = 98
snd_shield_hit_wood_wood = 99
snd_shield_hit_metal_metal = 100
snd_shield_hit_wood_metal = 101
snd_shield_hit_metal_wood = 102
snd_shield_broken = 103
snd_man_hit = 104
snd_man_die = 105
snd_woman_hit = 106
snd_woman_die = 107
snd_woman_yell = 108
snd_hide = 109
snd_unhide = 110
snd_neigh = 111
snd_gallop = 112
snd_battle = 113
snd_arrow_hit_body = 114
snd_metal_hit_low_armor_low_damage = 115
snd_metal_hit_low_armor_high_damage = 116
snd_metal_hit_high_armor_low_damage = 117
snd_metal_hit_high_armor_high_damage = 118
snd_wooden_hit_low_armor_low_damage = 119
snd_wooden_hit_low_armor_high_damage = 120
snd_wooden_hit_high_armor_low_damage = 121
snd_wooden_hit_high_armor_high_damage = 122
snd_mp_arrow_hit_target = 123
snd_blunt_hit = 124
snd_player_hit_by_arrow = 125
snd_pistol_shot = 126
snd_man_grunt = 127
snd_man_breath_hard = 128
snd_man_stun = 129
snd_man_grunt_long = 130
snd_man_yell = 131
snd_man_warcry = 132
snd_encounter_looters = 133
snd_encounter_bandits = 134
snd_encounter_farmers = 135
snd_encounter_sea_raiders = 136
snd_encounter_steppe_bandits = 137
snd_encounter_nobleman = 138
snd_encounter_vaegirs_ally = 139
snd_encounter_vaegirs_neutral = 140
snd_encounter_vaegirs_enemy = 141
snd_sneak_town_halt = 142
snd_horse_walk = 143
snd_horse_trot = 144
snd_horse_canter = 145
snd_horse_gallop = 146
snd_horse_breath = 147
snd_horse_snort = 148
snd_horse_low_whinny = 149
snd_block_fist = 150
snd_man_hit_blunt_weak = 151
snd_man_hit_blunt_strong = 152
snd_man_hit_pierce_weak = 153
snd_man_hit_pierce_strong = 154
snd_man_hit_cut_weak = 155
snd_man_hit_cut_strong = 156
snd_man_victory = 157
snd_fire_loop = 158
snd_torch_loop = 159
snd_dummy_hit = 160
snd_dummy_destroyed = 161
snd_gourd_destroyed = 162
snd_cow_moo = 163
snd_cow_slaughter = 164
snd_distant_dog_bark = 165
snd_distant_owl = 166
snd_distant_chicken = 167
snd_distant_carpenter = 168
snd_distant_blacksmith = 169
snd_arena_ambiance = 170
snd_town_ambiance = 171
snd_tutorial_fail = 172
snd_your_flag_taken = 173
snd_enemy_flag_taken = 174
snd_flag_returned = 175
snd_team_scored_a_point = 176
snd_enemy_scored_a_point = 177





####################################################################################################################
#  Each quest record contains the following fields:
#  1) Info page id: used for referencing info pages in other files. The prefix ip_ is automatically added before each info page id.
#  2) Info page name: Name displayed in the info page screen.
#
####################################################################################################################

info_pages = [
 ("morale", "Morale", "Morale represents the ability and willingness of the troops in a party to summon up the endurance, bravery, and discipline they need to face the stresses of battle and the march. It is not the same thing as the troops' happiness. Elite troops may grumble and whine about the hardships of campaigning -- but then stand together as one when the arrows start to fly. On the other hand, a commander who gives his men everything they want may find that they grow soft, and waiver before the enemy's charge.^^ Morale's greatest impact is on a party's behavior in battle, determining how aggressively troops engage the enemy, and how likely they are to break and run if they perceive the tide of battle turning against them. Morale also affects a party's march speed, as a less motivated party will move more slowly, as the men are not pushing themselves to their physical limit, and pause more frequently, as it waits for stragglers to catch up. Finally, a party with very low morale will start to suffer desertions.^^ Some factors that affect morale are intuitive. For example, a charismatic commander with a reputation for winning battles can infuse his or her men with a sense of confidence. Leaders who give their men well ample and varied supplies of food, and pay them on time, demonstrate that they care about their troops' welfare, and are less likely to lead them into disaster.^^ Other factors are less intuitive -- particularly those related to a party's sense of group cohesion. In a small tight-knit party, for example, men will often fight hard against daunting odds to avoid showing cowardice before their comrades-in-arms. A large party on the other hand may see its cohesion strained, as the commander has less time to supervise the men, listen to their grievances, and resolve their disputes. Frequent battles will strengthen the bonds between men, while long periods without combat will see the troops become bored and quarrelsome.^^ The morale report, accessibly by hitting the 'reports' button will give the player a sense of the factors affecting his or her men's morale."),
 ("economy", "Economy", "Towns and villages in Calradia need a wide variety of goods for their populations to remain healthy and productive. First in importance is food. Grain is the staple crop of Calradia, but people also need fat and protein in the form of meat, fish, or cheese. It takes almost as much work to preserve meat as to produce it in the first place, so salt is also in high demand. After food comes clothing: heavy wool, lighter linens, or luxurious velvet. Finally, people need the tools of their trade: ironware, pottery, leatherware, and, of course, arms, armor, and horses for war.^^Most agricultural products are produced in the villages, while artisans in the towns specialize in manufactured or artisanal goods like fabrics or ironware. Also, different resources can be found in different parts of the country. Consequently, the key to prosperity in Calradia is trade -- both between the villages and the towns, and between the major towns themselves.^^When trade flows, goods will be available and affordable, the population of a center will be healthy and energetic, and migrants will flock from the nearby regions. The center will produce more, consume more, and be able to contribute more in taxes to their lords. When trade dries up, towns and villages will see their workers flee to seek work elsewhere, and economic activity will drift to a stand-still. Thus, it is in the interests of rulers to protect trade routes from the hazards of war and banditry. A smart merchant, however, may want to seek out towns which have become isolated from the rest of the land, as he or she may be able to turn a tidy profit from the resulting price imbalances.^^Because villagers usually plan to take their goods to market in towns, village markets will be rather quiet places, and villagers will buy cheap and sell dear. Serious merchants will stick to the towns to make a profit, although some parties may decide to make a quick stop in a village to acquire supplies.^^A player who wants to know about the factors affecting a region's prosperity can speak to the guildmaster of the local town. Other information can be gleaned from passers-by, although they might not know very much outside of their own particular trade."),
 ("courtship", "Courtship", "Players may wish to marry into one of Calradia's noble families. Marriage is not necessary for a player to rise in power and stature, but it does provide players with an opportunity to improve their relation with lords and establish a claim to the throne.^^Marriage requirements will be different for males and females. A male character will usually need to pursue a traditional path of courtship. He should establish a reputation in Calradian aristocratic society, get on good terms with his bride's parents or guardians, and then woo the lady according to local custom. If a player grows impatient, he may attempt to take a shortcut -- but there will be consequences in his relations with other lords.^^A male character should keep in mind that other lords will be competing with him for the affections of the kingdom's ladies. Also, a lady's tastes are unpredictable, and a player may also find that the object of his love does not love him in return. Romance, in Calradia as elsewhere, does not always prosper. Of course, a player may resort to other, less gentlemanly means of winning a lady's heart, but again, that will have a serious impact on his reputation.^^To get started on the path of courtship, a male player should try to get involved in the social life of the Calradian aristocracy, attending feasts and tournaments. Also, wandering troubadours and poets can serve as a useful repository of information on courtship, and keep the player up to date about the latest gossip.^^Female characters can also marry -- but they should keep in mind that Calradian society is very traditional, and, as adventurers, they have chosen a very unconventional path for a woman. A female character may have to look for a while to find a lord who is open-minded enough to marry her.^^On the bright side, a female character does not have to go through the elaborate rituals of courtship, and she also may gain more from a marriage than her male counterpart. For a woman adventurer, marriage can be a quick path to power -- and an unscrupulous character may be able to use her husband as a tool of her political ambitions."),
 ("politics", "Politics", "The realms of Calradia, although they represent different cultures, all adhere to the same basic political system: feudalism. Feudalism is based on the relationships between individuals: the oaths of loyalty given by a vassal to his or her liege. In exchange for this oath, the vassal will usually receive a fief, a parcel of land whose income will be used by the vassal to raise troops to support the liege in time of war. A liege also has an obligation to protect his vassals, and to treat them justly.^^This is how it works in theory, anyway. In practice, vassals will not always work in their factions' interests, particular as they are often quarreling with one another. Nobles have different personalities, and sometimes those personalities clash. Or, perhaps two nobles were once friends, but fell out over in the aftermath of a setback or a defeat -- or because they both were wooing the same lady. Jealousies will also surface as they vie for the favor of the king -- perhaps over newly conquered lands, or over who will be given the coveted office of marshal, the lord in charge of organizing large-scale campaigns.^^When one realm in Calradia makes war on another, the political unity of the each kingdom is as important as the quality or number of its soldiers in determining the outcome. In a cohesive kingdom, nobles will join together in a large force to sweep their opponents before them. In a kingdom divided by petty quarrels, lords will fail to respond to the marshal's summons, or drift away to attend to their own business if a campaign is not going well. A faction's political cohesion will also impact warfare when campaigns are not in progress. In a divided faction, lords will be less likely to join together on raids and patrols, and come to each other's defense.^^If it seems self-defeating for nobles to bicker and quarrel when the enemy is just over the horizon, keep this in mind -- ultimately, a noble's loyalty goes not to a particular faction or culture, but to himself and to his family. If a noble fears that his faction is collapsing, or if he is being neglected by his liege, he can usually find a reason to withdraw his oath of allegiance, and change sides. Players should keep this in mind, as they may find that there are opportunities to turn discontented former enemies into allies."),
 ("character_backgrounds", "Character_Backgrounds", "A player character in Calradia may choose to come from a variety of social backgrounds. This choice will affect not just his or her starting skills and equipment, but also the course of his or her career as an adventurer.^^War and politics in Calradia are traditionally dominated by male aristocrats. A nobleman player character may find that he is invited into this 'old boys' club' fairly quickly, but women and commoners may face a few extra hurdles on the way. If you choose to start the game as a male nobleman, you can think of it as the 'easy' setting. Starting as a noblewoman or a male commoner is somewhat more difficult, and starting as a female commoner is probably the most challenging way to begin a game.^^However, women have some starting advantages. Simply by taking up arms, a female warrior will draw attention to herself, and she may find that she can build up her reputation faster than a male. Also, it is traditionally easier for a woman to marry up the social ladder than it is for a man, and a woman may find she can gain more from a strategic marital alliance than her male counterpart.^^Finally, keep in mind that the game does not place any limits on the upward mobility of characters based on their background. Noble or common, male or female, married or unmarried -- anyone can rise to become ruler of all Calradia, if they are sufficiently brave, lucky, or resourceful."),
 ("military_campaigns", "Military Campaigns", "When kingdoms in Calradia go to war, their armies have two basic offensive options. They can try to attack villages and lay waste to the countryside, damaging their enemy's prestige and economy. Or, they can try to seize and hold castles or towns, taking territory This second option can involve long, bloody sieges, but will yield more decisive results.^^It is important to note that the realms of Calradia do not field standing armies, which remain in the field as long as the ruler desires. Rather, Calradian realms are protected by feudal levies comprised of the major nobles and their individual retinues.  Sometimes, these nobles launch their own private attacks into enemy territory, but the most decisive events will usually take place when the great hosts are assembled. The kingdom's marshal, a noble appointed by the king, will summon the host before the campaign and lead them out to battle. However, he should be careful not to keep them in the field too long. Otherwise, the host will begin to disintegrate, as the vassals drift off to pursue their own business, and the army will be vulnerable to a counter-attack.^^For this reason, the rhythm of wars in Calradia often resemble the rhythm of a duel between two individual combatants. One side will gather its strength and seek to land a blow against the enemy's territory. If the marshal spends too little time gathering the vassals, he may not be able to do any real damage. If he spends too much time, then the campaign may end before it has even begun. A large realm will have an advantage over a smaller one, just as a brawny combatant has an edge over a smaller foe, but a realm's political cohesion can also be a factor, just as a fighter with great stamina can outlast her opponent. Sometimes, the armies of two realms will meet head on, resulting in a major battle in which both numbers and morale will decide the outcome.^^Kingdoms will have imperfect intelligence about their enemies. Attacking lords will need to frequently scout enemy territory to determine which fortresses may be vulnerable. An army defending its homeland will benefit from the alarms raised by castles and towns, which broadcast intelligence about enemy movements in the area. Such intelligence will be imprecise, however, particularly when it comes to numbers. A defending force which sets out to raise a siege or rescue a village may be able to overwhelm an unprepared attacker -- or it may miscalculate, and find that it is the one to be overwhelmed. Attackers, in turn, must be careful how far they advance into enemy territory, with aggressive marshals venturing further than cautious ones.^^Players will be expected to join in their faction's military campaign, either by joining the host, or by scouting ahead into enemy territory. Some players may find that their realm's marshal is too cautious, or too aggressive, for their tastes. In this case, they can intrigue with other lords to try to replace the marshal, or build support to become the marshal themselves.^^Most wars are of limited duration. A king who goes to war will, for the sake of honor, feel obligated to pursue the conflict for a short while. However, unless he is soundly beating his enemy. he may soon start looking for a way out of the conflict, lest he leave himself vulnerable to an attack by a third party. Calradia's rulers are keenly aware that today's ally may be tomorrow's enemy, and vice versa."),
]




################################################
###
### 处理器：命令操作列表
### 方便操作而已
###
################################################
from smart_module.smart_core.base.meta_command.meta_command_grammar_sugar_config import grammarSugarConfig

append = "@Append"
prepend = "@Prepend"
replace = "@Replace"
delete = "@Delete"




gameInitScript = "@GameInitScript"
gameStartMenu = "@GameStartMenu"


if __name__ == '__main__':
    print(grammarSugarConfig)


# -*- coding: utf-8 -*-
from smart_module.module_system.header_dialogs import *
from smart_module.module_system.module_scripts import *
from smart_module.smart_core.smart_module_slot import smartModuleSlotManager

'''
    巡逻队功能

    每48（由patrol_update_interval决定）小时会更新巡逻队（补充士兵，添加经验，贩卖俘虏）
    此巡逻队只为据点生成，包括，城镇，城堡，和村庄.
    巡逻队的数量取决于领主的荣誉，每200个荣誉会创建一只巡逻队，士兵的数量（统御,繁荣度），军事强度（战术和教练，繁荣度）
    
    如果据点没有领主，就不会创建巡逻队

    人数超过限制和金钱（据点的钱）不够时就不会再补充士兵，如果超过是不限制的。
    队数如果少于限制数量，就会创建以补足数量。
    士兵每天都会有经验加成
    俘虏贩卖金币会保存到据点

'''


## 巡逻队多久更新一次（游戏中的单位：小时）
patrol_update_interval = 48

## 多少个荣誉会生成一个巡逻队
patrol_count_per_party_renown= 500

## 每80个繁荣度会多增加一队巡逻队
patrol_center_prosperity_per_party=80

## 巡逻队基础队数,至少有一队(有领主时，最少队数)
patrol_base_count = 1

## 巡逻队基础数量（有领主时，最少人数）
patrol_base_size = 15


## 巡逻队基础军事强度（有领主时，最少军事强度）
patrol_base_strength = 2

## 每一点统御增加士兵数量
patrol_size_per_leadership = 5

## 据点每5繁荣度多增加一个士兵，slot_town_prosperity
patrol_center_prosperity_per_one = 5

## 每一点战术增加军事强度
patrol_strength_per_tactics = 1

## 每一点教练增加军事强度
patrol_strength_per_trainer = 1

## 据点每25个繁荣增加一点军事强度
patrol_center_prosperity_per_strength = 25

## 每次升级士兵的花费
patrol_update_cost_money = 500

## 以下内容非游戏程序员不要修改

## 【slot】

## party slot

## 用于保存巡逻队的数量
slot_party_patrol_num     = smartModuleSlotManager.getPartySlotNo("slot_party_patrol_num")
## 用于保存巡逻的据点
slot_party_protect_center = smartModuleSlotManager.getPartySlotNo("slot_party_protect_center")

## 【args】

## 巡逻队类型
spt_patrol             = 7


patrolGuardParty = {
    "name":"PatrolParty",
    "enable":True,
    "version":"v2.0.0",
    "desc":'''
        巡逻队功能
    
        每48（由patrol_update_interval决定）小时会更新巡逻队（补充士兵，添加经验，贩卖俘虏）
        此巡逻队只为据点生成，包括，城镇，城堡，和村庄.
        巡逻队的数量取决于领主的荣誉，每200个荣誉会创建一只巡逻队，士兵的数量（统御,繁荣度），军事强度（战术和教练，繁荣度）
        
        如果据点没有领主，就不会创建巡逻队
    
        人数超过限制和金钱（据点的钱）不够时就不会再补充士兵，如果超过是不限制的。
        队数如果少于限制数量，就会创建以补足数量。
        士兵每天都会有经验加成
        俘虏贩卖金币会保存到据点
    ''',
    "dependentOn":["PartyBaseScripts"],
    "actions":[
        ("Append@strings",[
            ## 代码功能
            ("s5_s_patrol_party","{s5}'s patrol Party"),
        ]),
        ("GameInitScript",[
            ## 游戏开始时就创建循环队
            (call_script, "script_update_patrol_partys_for_all"),
        ]),
        ("Append@simple_triggers",[
            ## 定期创建和维护巡逻队
            (patrol_update_interval, [
                (call_script, "script_update_patrol_partys_for_all"),
            ]),
            ## 在据点变更领主时，巡逻队也相应变更阵营
            (1, [
                ## 转变阵营的据点的巡逻队也要转换阵营(1个小时检测一次)
                ## 如果能够在阵营转换时就检测应该效果最好（但是可能需要修改系统代码）
                (call_script, "script_update_all_patrol_party_faction"),
                ## 统计被打败的队伍,从slot中减去
            ]),
            ## 显示一个城镇巡逻队的信息(用于测试巡逻队是否正常刷出)
            # (1,[
            #     (display_message,"@---------------------------"),
            #     (try_for_parties,":party"),
            #         (party_is_active,":party"),
            #         (party_slot_eq,":party",slot_party_type,spt_patrol),
            #         (party_slot_eq,":party",slot_party_protect_center,"p_town_3"),
            #         (assign,reg1,":party"),
            #         ##(display_message,"@p_town_3 party id: {reg1}"),
            #         (party_get_num_companions,reg2,":party"),
            #         (display_message,"@p_town_3 id:{reg1} party size: {reg2}"),
            #     (try_end),
            #     (display_message,"@---------------------------"),
            # ]),
            ## 每6小时统计一次每一个据点巡逻队的数量，便于巡逻队被击败后及时创建
            ## ## 统计被打败的队伍,从slot中减去
            (6, [
                # (display_message,"@center patrol begin update size"),
                ## 初始化所有城镇的数量
                (try_for_parties, ":center_no"),
                (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_town),
                (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_castle),
                (party_slot_eq, ":center_no", slot_party_type, spt_village),
                (party_set_slot, ":center_no", slot_party_patrol_num, 0),
                (try_end),

                ## 统计据点巡逻队数量
                (try_for_parties, ":party"),
                (party_is_active, ":party"),
                (party_slot_eq, ":party", slot_party_type, spt_patrol),
                (party_get_slot, ":center", ":party", slot_party_protect_center),
                (party_get_slot, ":num", ":center", slot_party_patrol_num),
                (val_add, ":num", 1),
                (party_set_slot, ":center", slot_party_patrol_num, ":num"),
                (str_store_party_name, s1, ":center"),
                (assign, reg1, ":num"),
                # (display_message,"@center({s1}) patrol size({reg1}) update"),
                (try_end),
            ]),

            # (6,[
            #     (call_script, "script_give_center_to_lord", "p_town_16", "trp_player", 1),
            #     (call_script,"script_update_all_notes"),
            # ]),
        ]),
        ("Append@scripts",[
            ## 获得巡逻队的军事强度
            ("get_patrol_center_strength", [
                (store_script_param_1, ":center_no"),
                (assign, ":strength", 0),
                (party_get_slot, ":leader", ":center_no", slot_town_lord),
                (try_begin),
                (ge, ":leader"),
                (assign, ":strength", patrol_base_strength),
                ## 战术加成
                (store_skill_level, ":tactics", skl_tactics, ":leader"),
                (store_mul, ":tactics_strength", ":tactics", patrol_strength_per_tactics),
                (val_add, ":strength", ":tactics_strength"),
                ## 教练加成
                (store_skill_level, ":trainer", skl_trainer, ":leader"),
                (store_mul, ":trainer_strength", ":trainer", patrol_strength_per_trainer),
                (val_add, ":strength", ":trainer_strength"),
                ## 繁荣度加成
                (party_get_slot, ":prosperity", ":center_no", slot_town_prosperity),
                (store_div, ":prosperity_strength", ":prosperity", patrol_center_prosperity_per_strength),
                (val_add, ":strength", ":prosperity_strength"),
                (try_end),
                (assign, reg0, ":strength"),
            ]),

            ("get_patrol_ideal_size", [
                (store_script_param_1, ":center_no"),
                (party_get_slot, ":leader", ":center_no", slot_town_lord),
                (assign, ":size", 0),
                (try_begin),
                (ge, ":leader", 0),
                (assign, ":size", patrol_base_size),
                ## 统御加成
                (store_skill_level, ":leadership", skl_leadership, ":leader"),
                (val_min, ":leadership", 1),
                (store_mul, ":leadership_size", ":leadership", patrol_size_per_leadership),
                (val_add, ":size", ":leadership_size"),
                ## 繁荣度加成
                (party_get_slot, ":prosperity", ":center_no", slot_town_prosperity),
                (store_div, ":prosperity_size", ":prosperity", patrol_center_prosperity_per_one),
                (val_add, ":size", ":prosperity_size"),
                (try_end),
                (assign, reg0, ":size"),
            ]),

            ("get_patrol_ideal_party_count", [
                (store_script_param_1, ":center_no"),
                (party_get_slot, ":leader", ":center_no", slot_town_lord),
                (assign, ":count", 0),
                (try_begin),
                (ge, ":leader", 0),
                (assign, ":count", patrol_base_count),
                ## 荣誉加成
                (troop_get_slot, ":renown", ":leader", slot_troop_renown),
                (store_div, ":renown_count", ":renown", patrol_count_per_party_renown),
                (val_add, ":count", ":renown_count"),
                ## 繁荣度加成
                (party_get_slot, ":prosperity", ":center_no", slot_town_prosperity),
                (store_div, ":prosperity_count", ":prosperity", patrol_center_prosperity_per_party),
                (val_add, ":count", ":prosperity_count"),
                (try_end),
                (assign, reg0, ":count"),
            ]),
            ## 获得巡逻队需要创建的数量
            ("get_patrol_center_need_create_num", [
                (store_script_param_1, ":center_no"),
                (party_get_slot, ":center_patrol_num", ":center_no", slot_party_patrol_num),
                (call_script, "script_get_patrol_ideal_party_count", ":center_no"),
                (assign, ":ideal_count", reg0),
                (store_sub, ":need_create_party_num", ":ideal_count", ":center_patrol_num"),
                (assign, reg1, ":need_create_party_num"),
            ]),
            ## 获得巡逻队需要招募的士兵数量
            ("get_patrol_party_need_size", [
                (store_script_param_1, ":party_no"),
                (party_get_slot, ":center_no", ":party_no", slot_party_protect_center),
                (call_script, "script_get_patrol_ideal_size", ":center_no"),
                (assign, ":ideal_size", reg0),
                (party_get_num_companions, ":cur_size", ":party_no"),
                (store_sub, ":need_size", ":ideal_size", ":cur_size"),
                (assign, reg0, ":need_size"),
            ]),

            ## 巡逻队的核心功能，驱动巡逻队运行的代码
            ("update_patrol_partys_for_all", [
                ## 领地巡逻队
                (try_for_range, ":center_no", centers_begin, centers_end),
                    (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_town),
                    (this_or_next | party_slot_eq, ":center_no", slot_party_type, spt_castle),
                    (party_slot_eq, ":center_no", slot_party_type, spt_village),
                    ## 排除玩家领地（玩家巡逻队需要玩家自己创建）
                    (neg | party_slot_eq, ":center_no", slot_town_lord, "trp_player"),

                    ## 计算每个巡逻队的军事强度
                    (call_script, "script_get_patrol_center_strength", ":center_no"),
                    (assign, ":strength", reg0),
                    ## 计算需要创建的部队数量
                    (call_script, "script_get_patrol_center_need_create_num", ":center_no"),
                    (assign, ":need_create_party_num", reg1),
                    (try_for_range, ":unused", 0, ":need_create_party_num"),
                    ## 在指定地方创建巡逻队
                    (call_script, "script_create_party", -1, ":center_no", -1, -1, "str_s5_s_patrol_party", spt_patrol,
                     "icon_khergit_horseman_b", -1),
                    (assign, ":party", reg0),
                    ## 设置巡逻据点
                    (party_set_slot, ":party", slot_party_protect_center, ":center_no"),
                    ## 增加士兵
                    (call_script, "script_party_add_members", ":party", -1, ":strength", -1, 30, 50),
                    ## 增加经验
                    (call_script, "script_party_add_xp_and_upgrade", ":party", ":strength", 50),
                    ## 设置ai
                    ##(call_script, "script_party_set_ai_state", ":lady_party",  spai_patrolling_around_center, ":home"),
                    ##(call_script,"script_party_change_ai_state",":party",ai_bhvr_patrol_party,":center_no",5),
                    (call_script, "script_set_party_ai_patrol_center", ":party", ":center_no",20),
                    # (call_script,"script_party_set_ai_state",":party",spai_patrolling_around_center,":center_no"),
                    # (str_store_party_name,s1,":center_no"),
                    # (display_message,"@create :{s1} patrol party"),
                    ## 更新创建的队数量，以免重复创建
                    (party_get_slot, ":center_patrol_num", ":center_no", slot_party_patrol_num),
                    (val_add, ":center_patrol_num", 1),
                    (party_set_slot, ":center_no", slot_party_patrol_num, ":center_patrol_num"),
                    (try_end),
                    (try_end),
                    ## 维护巡逻队
                    ## 1.处理俘虏（本国招降，它国贩卖）
                    ## 2.补充士兵(不会一次补足)
                    ## 3.升级士兵
                    (try_for_parties, ":party_no"),
                    (ge, ":party_no", 0),
                    (party_slot_eq, ":party_no", slot_party_type, spt_patrol),
                    ## 【处理俘虏】,招募本国，它国售卖
                    (call_script, "script_party_handle_prisoners", ":party_no", 0),
                    ## 【补充士兵】
                    ## 获得巡逻的据点
                    (party_get_slot, ":center_no", ":party_no", slot_party_protect_center),
                    ## 获得需要招募的数量
                    (call_script, "script_get_patrol_party_need_size", ":party_no"),
                    (assign, ":need_size", reg0),
                    (try_begin),
                    (gt, ":need_size", 0),
                    (party_slot_ge, ":center_no", slot_town_wealth, patrol_update_cost_money),
                    ## 花钱招募
                    (call_script, "script_update_center_wealth", ":center_no", patrol_update_cost_money, -1),
                    ## 增加士兵
                    (call_script, "script_party_add_members", ":party_no", -1, ":strength", -1, 30, 50),
                    (try_end),
                    ## 【升级士兵】
                    ## 获得巡逻据点的军事强度
                    (call_script, "script_get_patrol_center_strength", ":center_no"),
                    (assign, ":strength", reg0),
                    ## 增加经验
                    (call_script, "script_party_add_xp_and_upgrade", ":party_no", ":strength", 50),
                (try_end),
            ]),
            ## 转变阵营的据点的巡逻队也要转换阵营
            ("update_all_patrol_party_faction", [
                (try_for_parties, ":party_no"),
                (party_slot_eq, ":party_no", slot_party_type, spt_patrol),
                (party_get_slot, ":center_no", ":party_no", slot_party_protect_center),
                (store_faction_of_party, ":center_faction", ":center_no"),
                (party_set_faction, ":party_no", ":center_faction"),
                (try_end),
            ]),
        ]),
        ("Replace@dialogs|party_encounter_hostile_defender&1:Surrender_or_die%^&3:party_encounter_hostile_ultimatum_surrender&4",[
            [anyone | plyr, "party_encounter_hostile_defender", [],
             "Surrender or die!", "party_encounter_hostile_ultimatum_surrender", [
                 # (display_message,"@Surrender or die!"),
                 (try_begin),
                 # (str_store_party_name,s1,"p_main_party"),
                 (assign, ":party", "$g_encountered_party"),
                 # (str_store_party_name,s2,":party"),
                 (store_faction_of_party, ":faction", ":party"),
                 # (str_store_faction_name,s3,":faction"),
                 # (display_message,"@player({s1}) is attack party({s2}) of faction({s3})."),
                 (call_script, "script_change_player_relation_with_faction_ex", ":faction", -1),
                 (try_end),
             ]
             ],
        ]),
    ],
    "internationals":{
        "cns":{
            "game_strings":[
                "str_s5_s_patrol_party|{s5}的 巡 逻 队",
            ]
        }
    }
}


skl_trade = 0
skl_leadership = 1
skl_prisoner_management = 2
skl_reserved_1 = 3
skl_reserved_2 = 4
skl_reserved_3 = 5
skl_reserved_4 = 6
skl_persuasion = 7
skl_engineer = 8
skl_first_aid = 9
skl_surgery = 10
skl_wound_treatment = 11
skl_inventory_management = 12
skl_spotting = 13
skl_pathfinding = 14
skl_tactics = 15
skl_tracking = 16
skl_trainer = 17
skl_reserved_5 = 18
skl_reserved_6 = 19
skl_reserved_7 = 20
skl_reserved_8 = 21
skl_looting = 22
skl_horse_archery = 23
skl_riding = 24
skl_athletics = 25
skl_shield = 26
skl_weapon_master = 27
skl_reserved_9 = 28
skl_reserved_10 = 29
skl_reserved_11 = 30
skl_reserved_12 = 31
skl_reserved_13 = 32
skl_power_draw = 33
skl_power_throw = 34
skl_power_strike = 35
skl_ironflesh = 36
skl_reserved_14 = 37
skl_reserved_15 = 38
skl_reserved_16 = 39
skl_reserved_17 = 40
skl_reserved_18 = 41


